/*******************************************************************/
/*                                                                 */
/*   Program Name :  adbitconv                                     */
/*                                                                 */
/*      Version:  0.01  2005-06-13                                 */
/*                                                                 */
/*      Copyright (c) 2004-2005 T.Kondo/NICT All Right Reserved    */
/*                                                                 */
/*******************************************************************/
/*
 ho|ukah{[hf[^t@C̃TvO[hiAD\jϊ

 
          T.Kondo 2004.5.21
               Ver 0.00 2004.5.21
               Ver 0.01 2005.6.12  update due to update in common header name

    点  adbitconv filename [adbit [outfile [secoffset [span]]]]

         filename -- f[^t@C
               adbit    -- VȂ`c\ iftHg͂Pj
               outfile -- f[^o̓t@C
                           ftHgfilename+'.adN'
                           Nadbit
               secoffset -- ϊJnX^[gItZbgibj
                           ftHg͂O
               span      -- ϊf[^ԁibj
                           ftHg͑S

*/
#define ADBITCONV_VERSION "Ver. 2005-06-12"

#include "libk5.h"      // common header for IP-VLBI utilities

typedef struct ipboard {
	int adbit;         // A/Drbg (1,2,4,8)
	int numch;         // ͂@(1,4)
	int sfreq;         // TvOg(kHz)(40,100,200,500,1000,2000,4000,8000,16000)
    long int seconds;    // Ǒoߕb
    unsigned char hd[8]; // ho{[hwb_[ 
    long int nbsec;       // Pbf[^̃oCgiwb_[܂ށj 
    long int bpos;       // ݊JnoCgʒuiOX^[gj
} IPBOARD_T;

typedef struct ip_vlbi_file
{
	unsigned char bit32[4];
	unsigned char bit64[8];
	unsigned char *ibuf;
    int *idata;
	int sfreq;
	int icnt;   //  current unit number in a second data 
	int numb;   //  # of bytes in unit # of samples
	int usampl;  // unit sample number
	int kpos;
	int imax;    //  # of units in a second data
	char* fname; //I[vĂt@C
	double fsample;  //TvOg(Hz)
	double tsample; // sampling period
	double unitp;   // usampl*tsampl
	int   adbit;  // ADrbg
	long int seconds; // 00h00m00šoߕb
	int hh, mm, ss;
	int spatn; // sync pattern    
	int syncok; // =1 for 32bit all 1 detection // not used
	int format; // =0 for old format , =1 for new format
	FILE *stream;
	fpos_t pos;
	int fh;
	int ch;    
	int numch;
	int ierr;
	double dtime;  // time at begin of unit
	double dtimest;  // time at begin of file data
	unsigned int bytesread;
    int vflag, vflag_old;  // byte slip flag  added on 2003-02-14
                           //  if header sync detection is suceed
                           //     set to 1.  else 0
                           //   1: sync detection success 
                           //   0: sync detection error (may be byte slip)
                           //   vflag_old is the previous status
    long int dtime_vflag, dtime_vflag_old; // time at vflag decision made
} ip_vlbi_file_t;

void ip_hd_make_ini(IPBOARD_T *ipb)
{
    /* IP-VLBI{[ho̓f[^wb_[ׂč쐬 */
    int n,fid,aid;
    long int w1;
    unsigned char b,c;
    double fsampl;
    /* sync pattern 1 set */
    for (n=0; n<4; n++){
       ipb->hd[n]=0xFF;
    }
    /* seconds set */
    w1=ipb->seconds;
    b=(unsigned char)((w1 & 0x10000) >> 16);
    ipb->hd[4]=(unsigned char)(w1 & 0xFF);
    ipb->hd[5]=(unsigned char)((w1 & 0xFF00) >> 8);
    b=(unsigned char)((w1 & 0x10000) >> 16);
    /* ch flag set */
    if( ipb->numch == 4){
        b=b | 0x2;
    }
    /* sampling freq index set */
    fid=0;
    //if( ipb->sfreq ==    40) fid=0;
    if( ipb->sfreq ==   100) fid=1;
    if( ipb->sfreq ==   200) fid=2;
    if( ipb->sfreq ==   500) fid=3;
    if( ipb->sfreq ==  1000) fid=4;
    if( ipb->sfreq ==  2000) fid=5;
    if( ipb->sfreq ==  4000) fid=6;
    if( ipb->sfreq ==  8000) fid=7;
    if( ipb->sfreq == 16000) fid=8;
    c=(unsigned char)(fid << 2);
    b=b | c;
    /* A/D bit length set */
    aid=0;
    //if( ipb->adbit == 1 ) aid=0;
    if( ipb->adbit == 2 ) aid=1;
    if( ipb->adbit == 4 ) aid=2;
    if( ipb->adbit == 8 ) aid=3;
    c=(unsigned char)(aid << 6);
    b=b | c;
    ipb->hd[6]=b;
    /* sync pattern 2 set */
    ipb->hd[7]=0x8B;
    /* internal data initialize */
    fsampl=1000.0*(double)ipb->sfreq; // sampling frequency (Hz)

    ipb->nbsec=(((long)fsampl)*ipb->adbit*ipb->numch)/8;  // # of bytes in 1 sec data except header
    ipb->nbsec+=8;
    ipb->bpos=0;
}

void ip_hd_make(IPBOARD_T *ipb)
{
    /* IP-VLBI{[ho̓f[^wb_[̂ݍXV */
    long int w1;
    unsigned char b,c;
    /* seconds set */
    w1=ipb->seconds;
    b=(unsigned char)((w1 & 0x10000) >> 16);
    ipb->hd[4]=(unsigned char)(w1 & 0xFF);
    ipb->hd[5]=(unsigned char)((w1 & 0xFF00) >> 8);
    b=ipb->hd[6];
    c=(unsigned char)((w1 & 0x10000) >> 16);
    b=(b & 0xFE) | (c & 0x1);
    ipb->hd[6]=b;
}

int checkheader(ip_vlbi_file_t *S)
{
       /* Read 64bit Data header check to get sampling parameters */
    /*
       if first 64bit data is not header block, found header block automatically
       return value > 0 means above condition occurred
    */
      
       int iok, i, n, i32;
       //fpos_t save;
       i32=0;

       S->vflag_old=S->vflag;                  // keep current valid flag
       S->dtime_vflag_old=S->dtime_vflag;  //keep time when vflag decesion made
       S->vflag=0;
       S->bytesread = fread( S->bit32, sizeof( char ), 4, S->stream );
       if( ferror( S->stream ) || S->bytesread==0 )
       {
           printf("checkheader: File EOF or Read Error! (%s)\n",S->fname);
	       i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }
       for (n=0; n<4; n++){
           S->bit64[n]=S->bit32[n];
       }

loop1:

       S->bytesread = fread( S->bit32, sizeof( char ), 4, S->stream );
       if( ferror( S->stream ) || S->bytesread==0 )
       {
           printf("checkheader: File EOF or Read Error! (%s)\n",S->fname);
	       i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }
       for (n=0; n<4; n++){
           S->bit64[n+4]=S->bit32[n];
       }
       headerchkn(S->bit64,&iok,&S->spatn,&S->adbit,&S->sfreq,&S->numch,
                        &S->seconds,&S->hh,&S->mm,&S->ss);
       if (iok == 0)   // sync detection
	   {
          if(i32 !=0) printf("  found after %d bits read!\n",i32*32);
          printf("checkheader: Header (64bits) Sync Detected!!\n");
		  printf("checkheader:   File : %s\n",S->fname);
          printf("checkheader:   A/D(bits) %d  CHs %d  SFreq(kHz) %d",
                    S->adbit,S->numch, S->sfreq);
          printf("  Time %02d:%02d:%02d  sec %d\n",
                    S->hh,S->mm,S->ss,S->seconds);
          S->format=1;          
          goto exit_loop;
	   }
       else
       {
           if(i32 ==0) {
               printf("checkheader: First 64bit is not header block!\n");
               printf("checkheader:   Data file name (%s)\n",S->fname);
               printf("checkheader:   searching header.....");
           }
           /* search sync pattern by shifting read position by 32 bits forward */
           for (n=0; n<4; n++){
               S->bit64[n]=S->bit64[n+4];
           }
           i32++;
       }
       goto loop1;
exit_loop:
       if(i32 == 0) S->vflag=1;
       S->dtime_vflag=S->seconds;
       //printf("checkheader:   vflag     dtime_vflag     %d %d\n",S->vflag,S->dtime_vflag);
       //printf("checkheader:   vflag_old dtime_vflag_old %d %d\n",S->vflag_old,S->dtime_vflag_old);
       return i32;
}

int fwdNsec(unsigned char* ibuf, int nsec, int numb, int imax,ip_vlbi_file_t *dat)
{
    int n, i;
    if(nsec <= 0) return 0;
	//printf("fwdNsec: nsec, numb, imax %d %d %d\n",nsec,numb,imax); //debug
    for(n=0; n<nsec; n++)
    {
        for(i=0; i< imax; i++)
        {
            dat->bytesread = fread( ibuf, sizeof( char ), numb, dat->stream );
            //printf("i, bytesread %d %d %d\n",i,dat->bytesread,ferror( dat->stream ));
            if( ferror( dat->stream ) || dat->bytesread==0 ){
		        return -1;
            }
		}
        if(checkheader(&(*dat)) < 0){
             return -1;
        }
     }
     return 0;
}


int conv4byte2int(unsigned char *b4)
{
    int i, k, iwork, iw2, ipick, flag, pflag;
    unsigned char iw;
    iwork=0;
    flag=0;  // flag to reverse bit order  1: reverse
    pflag=0;  // flag to change parity position   not used yet
    for(i=0; i<4; i++){
       iw=b4[i];
       iw2=(int)iw;
       for(k=0; k<8; k++){
           ipick=iw2 & 0x1;

           if(flag==0){
               iwork=iwork >> 1;
               ipick =ipick << 31;
               iwork= ipick | (iwork & 0x7FFFFFFF);
           } else {
               iwork=iwork << 1;
               ipick =ipick & 0x1;
               iwork= ipick | (iwork & 0xFFFFFFFE);
           }
           iw2=iw2 >> 1;
       }
    }
    return iwork;
}

int conv32bit2int(int *idat32)
{
    //int idat32[32];   // 32 bit data
    //int i4;   return 
    int i, iwork, ipick, flag;
    iwork=0;
    flag=0;  // flag to reverse bit order  1: reverse
    for(i=0; i<32; i++){
       ipick=idat32[i];
       if(flag==0){
           iwork=iwork >> 1;
           ipick =ipick << 31;
           iwork= ipick | (iwork & 0x7FFFFFFF);
       } else {
           iwork=iwork << 1;
           ipick =ipick & 0x1;
           iwork= ipick | (iwork & 0xFFFFFFFE);
       }
    }
    return iwork;
}


int main(int argc, char *argv[])
{
       int sfreq, adbit, numch, kpos;
       int adbitorg;
       long int startoffset; // start time offset in sec
       long int counts;        //sample counter
       long int counts2;        //extract sample counter
       long int sofst_pos;    // start sample position from sec tic
       int nloop;
       int i,imax, m, k, n;
       int smode,usampl;
       int obuf[1024];
       int nwrite=1024;
       int iwrite;
       int idat32[32];
       int idat,ipick;
       int ibcount;
       int opos, nrepeat;
       unsigned char *ibuf;
       int nbyte=5000;
       fpos_t pos;
       int bytesread;
       int bits, bytes, numb;
	   //int nch;
       int span,secc;
       double fsampl,tsampl,fact;
       double cfact; //conversion factor
       double dw;
       int *idata;
	   double soffset, fsec;
       char fname[80],oname[80];
       char lab2[60], ltime1st[60];
       char labfsampl[10];
       char dflt_oname[80];
       char* dflt_fname="tds.data";
       ip_vlbi_file_t X;
       FILE *fo;
       IPBOARD_T ipb;

       if(argc == 1){
          printf("adbitconv  %s\n",ADBITCONV_VERSION);
          printf("Usage: adbitconv filename [adbit [outfile [secofst [span]]]]\n");
          printf("\n");
          printf("         where  filename ---- Data file name\n");
          printf("                adbit   ---- New AD bits\n");
          printf("                             default is 1\n");         
          printf("                outfile ---- Extract data outfile name\n");
          printf("                             default is filename+.adN\n");         
          printf("                             where N is adbit\n");         
          printf("                secofst ---- Extract start time offset (sec)\n");
          printf("                             default is 0\n");         
          printf("                span    ---- Extract span (sec)\n");
          printf("                             default is all\n");         
          exit(0);
       }

       /* data file name get */
       strcpy(fname,*(argv+1));
       if (fname[0]=='0'){
          strcpy(fname,dflt_fname);
       }
       X.fname=fname;

       /* adbit to extract get */
       if (argc>=3)
       {
          adbit = atoi(*(argv + 2 ));
       } else {
          adbit = 1;   
       }
       /* default oname create */
       sprintf(dflt_oname,"%s.ad%d\0",fname,adbit);

       /* Out file name get */
       if (argc>=4)
       {
           strcpy(oname,*(argv+3));
       } else {
           strcpy(oname,dflt_oname);
       }

       /* start offset get */
       if (argc>=5)
       {
          soffset = atof(*(argv + 4 ));
       } else {
          soffset =0.0;   
       }

       /* span get */
       if (argc>=6)
       {
          span = atoi(*(argv + 5 ));
       } else {
          span =0;   
       }

       /* Run condition monitor for debug*/
       printf("Data File is %s\n",fname);
       printf("Out File is %s\n",oname);
       printf("New AD bit is  %d\n",adbit);

       /* Program Name Display */
       printf("*******************************************************\n");
       printf("* IP-VLBI DATA A/D RESOLUTION CONVERTER (adbitconv)   *\n");
       printf("*         %s    by  T.KONDO/NICT      *\n",ADBITCONV_VERSION);
       printf("*******************************************************\n");
       printf("\n");

       /* Data file open */
       if( (X.stream  = fopen( X.fname, "rb" )) == NULL )
       {
           printf("Data File %s   Open Error! \n",fname);
		   goto exitend2;
       }


       /* Data header check to get sampling parameters */
       if(checkheader(&X) < 0){
            goto exitend3;
       }

       /* AD bit check */

       if(adbit == X.adbit) {  // nothing to do
           printf("AD resolution in an data file is equal to given AD resolution (%d)\n",adbit);
           printf("So nothing to do!\n");
           goto exitend3;
       }

       /* output file create */
       if( (fo  = fopen( oname, "wb" )) == NULL )
       {
          printf("File  %s  Open Error! \n",oname);
          goto exitend3;
	   } 

       /* parameter set */
        sfreq=X.sfreq;
        adbitorg=X.adbit;
        numch=X.numch;
        fsampl=1000.0*(float)sfreq;   // sampling frequency (Hz)
        tsampl=1.0/fsampl;            // sampling period (sec)
        bits=((long)fsampl)*adbitorg*numch;  // # of bits in 1 sec data except header
        bytes=bits/8;                     // # of bytes in 1 sec data except header
        X.fsample=(float)fsampl;

        sofst_pos=0;
        //nrepeat=32/adbit;
        nrepeat=32/adbitorg;

        /* header parameters setup for output */
        ipb.adbit=adbit;  // new AD bits
        ipb.numch=numch;   
        ipb.sfreq=X.sfreq;
        ipb.seconds=X.seconds;
        ip_hd_make_ini(&ipb);


        //cfact=pow(2.0,adbit-adbitorg);
        cfact=(pow(2.0,adbit)-1.0)/(pow(2.0,adbitorg)-1.0);

        /* File read buffer size set */
        smode=1;                         // low resolution mode

        if(smode ==1){ usampl=40000; }
        else {
           usampl=50000;
           if(sfreq == 40){ usampl=40000; }
           if(sfreq == 100){ usampl=100000; }
           if(sfreq == 200){ usampl=200000; }
        }
        imax=(sfreq*1000)/usampl;
        //numb=usampl*adbit*numch/8;   // # of bytes in unit # of samples
        numb=usampl*adbitorg*numch/8;   // # of bytes in unit # of samples

        //printf("imax numb  %d %d\n",imax,numb);    //debug


        ibuf = (unsigned char *)malloc(numb);
        idata = (int *)malloc(4*usampl*sizeof(int));

        fact=((float)usampl)*((float)usampl);
        
        /* sampling frequency in text for print */
        if(sfreq < 1000){
            sprintf(labfsampl,"%dkHz\0",sfreq);
        } else {
            sprintf(labfsampl,"%dMHz\0",sfreq/1000);
        }
        //if(sfreq == 40){strcpy(labfsampl,"40kHz"); }
        //if(sfreq == 100){strcpy(labfsampl,"100kHz"); }
        //if(sfreq == 200){strcpy(labfsampl,"200kHz"); }
        //if(sfreq == 500){strcpy(labfsampl,"500kHz"); }
        //if(sfreq == 1000){strcpy(labfsampl,"1MHz"); }
        //if(sfreq == 2000){strcpy(labfsampl,"2MHz"); }
        //if(sfreq == 4000){strcpy(labfsampl,"4MHz"); }
        //if(sfreq == 8000){strcpy(labfsampl,"8MHz"); }
        //if(sfreq == 16000){strcpy(labfsampl,"16MHz"); }

       /* Data extract start point set */ 

       //soffset=0;  // if you need to give offset, change this
       startoffset=(long int)soffset;
       fsec=soffset-(double)startoffset;
       sofst_pos=(long int)(fsec/tsampl + 0.01);

       /* Data extraction period set */
       
    //   smpl_count=(long int)(period/tsampl + 0.01);

       /* Run condition display */

       //printf("startoffset, sofst_pos, smpl_count %d %d %d\n",startoffset,sofst_pos,smpl_count);
       printf("\n");
       printf("  Data File : %s\n",fname);
       printf("  Sampling Freq : %s\n",labfsampl);
       printf("  A/D bits      : %d\n",adbitorg);
       printf("  Number of CHs : %d\n",numch);

       printf("  New A/D bits  : %d\n",adbit);

       /* foward startffset seconds */
       if(fwdNsec(ibuf, startoffset, numb, imax, &X) <0) goto exitend;

       printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",X.hh,X.mm,X.ss,X.seconds);
       sprintf(ltime1st,"Time(UT) %02d:%02d:%02d\0",X.hh,X.mm,X.ss);
       strcpy(lab2,ltime1st);
        
       /* continue run info display */
       printf("  Start Time of Extraction : %02d:%02d:%02d.%08d\n",X.hh,X.mm,X.ss,(long int)(fsec*1.0e8));
     //  printf("  Period (sec)             : %f\n",period);
       printf("  Out File : %s\n",oname);

       /* header data update */
       ipb.seconds=X.seconds;
       ip_hd_make(&ipb);


       /* Data info out to out_file */
//       if(omode == 0) {
//          fprintf(fo,"**  Data File : %s\n",fname);
//          fprintf(fo,"**  Sampling Freq : %s\n",labfsampl);
//          fprintf(fo,"**  A/D bits      : %d\n",adbit);
//          fprintf(fo,"**  Number of CHs : %d\n",numch);
//          fprintf(fo,"**  Extracted CH# : %d\n",nch);
//          fprintf(fo,"**  Start Time of Data : %02d:%02d:%02d.%08d\n",X.hh,X.mm,X.ss,(long int)(fsec*1.0e8));
//         fprintf(fo,"**  Period (sec)       : %f\n",period);
//      }
       
       /* Data extraction start */
	   counts=0;
	   counts2=0;
       secc=0;
       //m=nch-1;
       nloop=0;
       ibcount=0;
       opos=0;
       obuf[opos]=conv4byte2int(ipb.hd);
       opos++;
       obuf[opos]=conv4byte2int(ipb.hd+4);
       opos++;

       /* Data read loop start here */

loop1:
        if(nloop >0) {
           if(checkheader(&X) < 0) goto exitend; 
         //   printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",X.hh,X.mm,X.ss,X.seconds);
           if(span != 0 && secc > span) goto exitend;
           secc++;
           /* header data update */
           ipb.seconds=X.seconds;
           ip_hd_make(&ipb);
           /* put K5 header data into obuf */

           obuf[opos]=conv4byte2int(ipb.hd);
           opos++;
           if(opos == nwrite){
                //printf("Debug %d\n",nloop);
                iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                opos=0;
           }
           obuf[opos]=conv4byte2int(ipb.hd+4);
           opos++;
           if(opos == nwrite){
                //printf("Debug %d\n",nloop);
                iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                opos=0;
           }
        }

        for(i=0; i< imax; i++)
        {
             bytesread = fread( ibuf, sizeof( char ), numb, X.stream );
             //printf("bytes read 2 %d\n",bytesread); //debug
             if( ferror( X.stream ) || bytesread==0 ) goto exithere; 
             parseADdata_2(ibuf,numb,adbitorg,idata, &kpos);
        	 //printf("adbit= %d\n",adbit);

             // convert AD resolution and file out every 8 bytes 

             //printf("numb, kpos, numch  %d %d %d\n",numb,kpos,numch);
             for(k=0; k<kpos/numch; k++)
             {
                 for(n=0; n< numch; n++){
                     counts++;
                     idat=idata[k*numch+n];
                     // convert AD bit resolution 
                     dw=(double)idat*cfact;
                     if(cfact < 1.0){
                         idat=(int)(dw+0.5);
                     } else {
                         idat=(int)dw;
                     }

                     for(m=0; m< adbit; m++){
                        ipick=idat & 0x1;
                        //printf(" %d",ipick);
                        idat32[ibcount]=ipick;
                        ibcount++;
                        idat=idat >> 1;
                     } 
                    if(ibcount==32){
                       obuf[opos]=conv32bit2int(idat32);
                       opos++;
                       if(opos == nwrite){
                           //printf("Debug %d\n",nloop);
                           iwrite=fwrite(obuf,sizeof(int),nwrite,fo);
                           opos=0;
                       } 
                       ibcount=0;
                    } 
                 } 
             }
       }
       nloop++;
       goto loop1;
exithere:
       printf("\n");
       printf("AD bit resolution conversion finished.\n");
       fgetpos(X.stream,&pos);  //debug
       //printf("\n %d bytes read\n",pos); 
       //counts2--;
       //printf("counts2 = %d\n",counts2);
       i=fclose( fo ) ;

exitend: 
       
       free(ibuf);
       free(idata);

exitend3: 
       i=fclose( X.stream ) ;

 exitend2: 

       return 0;
}
