/*************************************************************************/
/*                                                                       */
/*   Program Name :  autoobs                                             */
/*                                                                       */
/*     Autoobs carries out K5/VSSP(IP-VLBI) observations automatically   */
/*          according to a schedule file                                 */
/*                                                                       */
/*     Enviroment variables                                              */
/*           K5SKED     ---   default directory for schedule file        */
/*           K5OUTINFO   --   default output directory info file name    */
/*           K5FREQG     --   default frequency group of PC              */
/*           K5NAMING    --   default out file naming rule               */
/*           K5SUBNET    --   default subnet mode                        */
/*           K5SATKEY    --   default satellite keyword for 1ch obs mode */
/*                                                                       */
/*      Version:  0.00  2001-08-17                                       */
/*                2.21  2002-07-22                                       */
/*                3.01  2003-06-20                                       */
/*                3.02  2003-07-20  add env variable monitor             */
/*                3.10  2003-07-31  add naming rule selection            */
/*                3.11  2003-08-02  add K5SKED env variable              */
/*                3.20  2003-08-09  add subnet obs mode                  */
/*                3.21  2003-11-01  add K5SATKEY                         */
/*                3.22  2003-11-20  add file size non-limit mode         */
/*                3.23  2003-11-22  bug fix in Ver 3.22 (had a bug for   */
/*                                  4ch mode)                            */
/*                3.30  2003-11-27  Allow multi satellite keywords.      */
/*                                  Allow satellite keywords set in an   */
/*                                  run information file.                */
/*                                  Modify information displayed on CRT  */
/*                                  so as to give information about      */
/*                                  sampling mode of the next scan       */
/*                3.31  2004-05-21  Debug in K5SUBNET environment        */
/*                                  variable analysis.                   */
/*                                  Allow max 8 character satellite key  */
/*                3.40  2004-05-25  Reflect the updated "sampleget"      */
/*                3.50  2005-01-12  Allow the use of VEX file            */
/*                3.60  2005-02-04  More robust against disk full.       */
/*                                  Add naming Type-III which is         */
/*                                  compliant with "e-VLBI File-Naming   */
/*                                  Conventions (9 Nov 2004)"            */
/*                                  Ref ftp://web.haystack.mit.edu/pub/  */
/*                                    e-vlbi/filenaming_conventions.pdf  */
/*                3.61  2005-02-08  Slight modification on naming rule.  */
/*                                  identification of # of ch become     */
/*                                  possible in case of Type I and II    */
/*                3.62  2005-02-12  Debug in Ver.3.60 and 3.61           */
/*                3.63  2005-02-17  Debug in Ver.3.62                    */
/*                3.70  2005-06-05  Add -NOSATKEY command, and made      */
/*                                  program structure change             */
/*                3.71  2005-06-06  minor revision                       */
/*                3.72  2005-06-16  large scale restructuring made       */
/*                                  A bug in naming rule Type-III was    */
/*                                  fixed                                */
/*                3.73  2005-07-25  reflect "tape early start" parameter */
/*                                                                       */
/*      Copyright (c) 2002-2005 T.Kondo/NICT All Right Reserved          */
/*                                                                       */
/*************************************************************************/
 
/*

  K5/VSSP (IP-VLBI)  ItCϑvO

        uNOZOMIvϑ͂Pch[hAϑ͂Sch[hɎIɐ؂ւAfBXNeʂ
               ߖsB
               PF1chTv[{[hɂ4chTv[{[hch1ƓrfIM
               ͂邱ƁI
               QFû݁vϑ͓dg̍ŏ̂R"NOZ"ǂŔB
               ϐK5SATKEYł̃L[[hZbgiύXj\B
               CvO K5SATKEYDFLT ύX邱ƂŃftHg̉qɕύX
               IvV܂̓R}h-NOSATKEYgautoobs𑖂点ꍇ
               ׂẴL[[h͖ƂȂB

    点 ̂P

       autoobs sked_file [station_id [span [ sfreq [ adbit [numch
                      [outdir [y_or_n [naming_type [freqg [subnet 
                      [f_size_mode]]]]]]]]]]] [-NOSATKEY] [-NOEARLY]

           sked_file ---- XPW[t@C(SKED`܂VEX`j
                          擪'-'tƁAp[^t@Cirun information file)A
                          ȉ̃p[^"-NOSATKEY"ȊO͖ƂȂB

                staion_id ----  ID (XPW[t@CŋLqĂǂhc)
                span ---- ϑ (sec)
                          iXPW[t@CŋK肵Ă
                           ϑԂZړIj
                           ftHg͂OiXPW[t@Cʂ̊ϑԂ̈Ӂj
                sfreq  TvOg
                       40,100,200,500  (kHz)
                       1,2,4,8,16      (MHz)
                          ftHg16
                adbit  `c\irbg)
                       1,2,4,8
                          ftHg1
                numch  `l
                       1 ܂ 4
                          ftHg1
                outdir  f[^t@Co̓fBNg
                          擪'-'tƁAo̓fBNg؂ւt@CwB
                          ftHg̓JgfBNg
                y_or_n  ϑ͈͂𐧌ꍇ Y, ׂĊϑ̏ꍇN
                        ȗ͑Θbŉ񓚂
                naming_type o̓t@C
                       1܂-1: ^CvP  ]^CviftHgj
                           XDDDNNNN.[#ch.]dat  
                             
                              X    -- ǂhc(P) 
                              DDD  -- Ԗڂ̃XL̒ʓ
                              NNNN -- ϑԍ  (S)
                              #ch  -- `l (1 ܂ 4) (^Cv̏ꍇ̂݁j
                       2܂-2: ^CvQ  K5/VSSPŗL^Cv
                            sidDDDHHMMSSG.[#ch.]dat
                             
                               sid -- ID (PQ)
                               DDD -- XLJnʓiRj
                               HH  -- XLJniQj
                               MM  -- XLJniQj
                               SS  -- XLJnbiQj
                               G   -- gO[vID (a|b|c|d) ܂
                                       null
                               #ch -- `l (1 ܂ 4)(^Cv̏ꍇ̂݁j
                       3: ^CvR  e-VLBIt@C
                            expid_sidG_scanid_YYYYDDDDHHMMSS.k5
                              
                                expid -- R[h
                                sid   -- ǂhciQj啶̏ꍇ͏ɕϊ
                                G     -- ^[~i(PC)ID (1|2|3|4) ܂nullBfreqgp[^Ŏw
                                scanid -- XLhc
                                          VEXt@Cgp̓XLhĉ܂
                                          SKEDgpddd-hhmm  ̒ɕ̊ϑ
                                         ꍇ͂Qڈȍ~ɎԏɍŌa,b,c,d,... Ă
                                       ==>ꍇ͎ԏɍŌa,b,c,d,... Ă
                                YYYYDDDHHMMSS -- ϑJn
                                .k5  --- K5f[^ʎq
                       4: gp
                       5: ^CvT
                           expid_scan#.stcode.k5a(-d)
                               expid   -- R[h
                                     scan#   -- XLԍ
                                     stcode  -- ǃR[h
                                     k5  ------ Œl
                                     a-d   ---- gO[v
                freqg   t@C^CvQAR܂͂TŎgpgO[vhc
                        ܂PCIDB^CvRł͐̂݁B
                        (1|2|3|4)܂(a|b|c|d)iftHgnullj
                subnet   --  0: subnet mode off  1:subnet on (default)
                f_size_mode -- 0: no limitation  1: divided into each 2GB size(default)

             -NOSATKEY  --- @ȂTeCgL[̐ݒƂ
             -NOEARLY   --- "tape early start"p[^𖳌Ƃ

    点 ̂Q
        autoobs sked_file [option]
          autoobs [option]

            optionisj͈ȉ̂Ƃ

             -run runparam_file --- p[^t@C̎w
                          ̎wsƂ͈ȉ̃p[^
                          "-NOSATKEY"ȊO͖ƂȂB

             -sked sked_file ---- XPW[t@C(SKED`܂VEX`j
                                   ̎w

             -sid staion_id ----  ID ̃Zbg
             -span span ---- ϑ (sec)̃Zbg
                          iXPW[t@CŋK肵Ă
                           ϑԂZړIj
                           ftHg͂OiXPW[t@Cʂ̊ϑԂ̈Ӂj
             -sfreq 40|100|200|500|1|2|4|8|16   TvOg
                       40,100,200,500  (kHz)
                       1,2,4,8,16      (MHz)
                          ftHg16
             -ad 1|2|4|8  `c\irbg)
                       1,2,4,8
                          ftHg1
             -nch 1|4  `l
                       1 ܂ 4
                          ftHg1
             -odir outdir  f[^t@Co̓fBNg̃Zbg
             -dinfo dirinfo_file  o̓fBNg؂ւt@C
             -nolimit  ϑ͈͂𐧌ȂiׂĊϑsj
                        ȗ͑Θbŉ񓚂
             -naming 1|-1|2|-2|3 o̓t@C̃Zbg
                       1܂-1: ^CvP  ]^CviftHgj
                           XDDDNNNN.[#ch.]dat  
                             
                              X    -- ǂhc(P) 
                              DDD  -- Ԗڂ̃XL̒ʓ
                              NNNN -- ϑԍ  (S)
                              #ch  -- `l (1 ܂ 4) (^Cv̏ꍇ̂݁j
                       2܂-2: ^CvQ  K5/VSSPŗL^Cv
                            sidDDDHHMMSSG.[#ch.]dat
                             
                               sid -- ID (PQ)
                               DDD -- XLJnʓiRj
                               HH  -- XLJniQj
                               MM  -- XLJniQj
                               SS  -- XLJnbiQj
                               G   -- gO[vID (a|b|c|d) ܂
                                       null
                               #ch -- `l (1 ܂ 4)(^Cv̏ꍇ̂݁j
                       3: ^CvR  e-VLBIt@C
                            expid_sidG_scanid_YYYYDDDDHHMMSS.k5
                              
                                expid -- R[h
                                sid   -- ǂhciQj啶̏ꍇ͏ɕϊ
                                G     -- ^[~i(PC)ID (1|2|3|4) ܂nullBfreqgp[^Ŏw
                                scanid -- XLhc
                                          VEXt@Cgp̓XLhĉ܂
                                          SKEDgpddd-hhmm  ̒ɕ̊ϑ
                                         ꍇ͂Qڈȍ~ɎԏɍŌa,b,c,d,... Ă
                                      ==>ꍇ͎ԏɍŌa,b,c,d,... Ă
                                YYYYDDDHHMMSS -- ϑJn
                                .k5  --- K5f[^ʎq
                       4: gp
                       5: ^CvT
                            expid_scan#.stcode.k5a(-d)
                               expid   -- R[h
                                     scan#   -- XLԍ
                                     stcode  -- ǃR[h
                                     k5  ------ Œl
                                     a-d   ---- gO[v
             -freqg 1|2|3|4(܂ a|b|c|d)   t@C^CvQAR܂͂TŎgpgO[vhc
                        ܂PCIDB^CvRł͐̂݁B
                        (1|2|3|4)܂(a|b|c|d)iftHgnullj
             -subnet 0|1   --  0: subnet mode off  1:subnet on (default)
             -fsize 0|1 -- 0: no limitation  1: divided into each 2GB size(default)
             -NOSATKEY  --- @ȂTeCgL[̐ݒƂ
             -NOEARLY   --- "tape early start"p[^𖳌Ƃ


    ϐ̃j^[
      autoobs env

                        
      ϐ
          K5SKED   --- XPW[t@CTfBNg
          K5LOGDIR --- Ot@Co͂fBNg
                       ftHg̓JgfBNg
          K5OUTINFO -- o̓fBNgt@C
          K5NAMING --- ϑt@C@1|2|3  
                      io[WƂ̌݊̂߁@old | new LAꂼ@1|2@ɑj 
          K5FREQG  --- ob̎󂯎gO[v@1|2|3|4 ܂́@a|b|c|d
                       (^CvR̖̎a,b,c,d͎I1,2,3,4ɖ|󂳂j
          K5SUBNET --- ϑ̃Tulbg[h on | off
          K5SATKEY --- 1chϑsqR[h@ "HYB"  "HYB GEO NOZ"

     FVC++ŃRpCENۂ̓X^bNTCY4MBȏ\񂷂邱Ɓilibvexgp͕Kj

*/

//#define TEMP_TEST    // for test in the case of disk full simulation

#define AUTOOBS_VERSION "Ver 3.73    2005-07-25"

#include "libk5.h"        // header for library "libk5"
                          // library "libk5" consists of libipvlbi.c and libskdvex.c

#ifdef WIN32
#define SIMULATION       // IP-BOARD simulation mode ON
#endif

//#ifndef WIN32
//#include <sys/tdsio.h>  // IP-VLBI board header  (now in libipvlbi_board_dummy.c)
//#endif


#ifdef SIMULATION
#include "libipvlbi_board_dummy.h"  // library for simulating IP-VLBI sampler board
#else
#include "libipvlbi_board.h"  // library for IP-VLBI sampler board
#endif

#define SLEEP_USEC 10000    // watch dog timer sep (10usec)
#define SLEEP_MSEC 10       // watch dog timer sep (10msec)

#define GET_TIME_FROM_BOARD 1  // Time source for automatic observation
                               //  1: taken from IP-VLBI board,  0: Taken from PC timer
#define LOGFILE_TYPE 0   // log file name type
                         //  0: expcode+stationid+"."+hostname+".log"      before Ver 3.60
                         //  0: expcode+"_"+stationid+"."+hostname+".log"  from ver 3.60
                         //  1: YYYY+hostname+".log"
                         //       where  YYYY is 4 digit year
                         //  -1: no log

#define EXPCODE_LOW_CASE   // convert experiment code into lower-case char in case of naming type 3
#define STAT_ID2_LOW_CASE   // convert 2char station code into lower-case char in case of naming type 3

//#define USE_STAT_ID2        // use 2 char station id for all processing (not implemented yet)

/* environment variables definition */
#define K5SKED    "K5SKED"     // dfault directory for schedule file
#define K5OUTINFO "K5OUTINFO"  // dfault for output directory info file name
#define K5LOGDIR  "K5LOGDIR"   // default for log out directry
#define K5FREQG   "K5FREQG"    // default frequency group of PC  (1|2|3|4 or a|b|c|d)
#define K5NAMING  "K5NAMING"   // default out file naming rule  "old" | "new"  
                               //         or "1" | "2" | "3" 
#define K5SUBNET  "K5SUBNET"   // default subnet mode  "on" | "off"  
#define K5SATKEY  "K5SATKEY"   // default satellite code to switch to 1ch mode  

/* default used in program set */
#define K5SKEDDFLT    "./"             // dfault directory for schedule file
#define K5OUTINFODFLT "k5outinfo.txt"  // default out directories 2nd(1st), 3rd(2nd),....
#define K5LOGDIRDFLT  "./"             // default log file directory
#define K5FREQGDFLT   "null"           // default frequency group (NULL)
//#define K5NAMINGDFLT  "old"            // default file naming rule
#define K5NAMINGDFLT  "1"            // default file naming rule
#define K5SUBNETDFLT  "on"             // default subnet mode
#define K5SATKEYDFLT  "HYB NOZ GEO"            // default satellite code for 1ch mode

logval_t lg;     // structure variable for log

typedef struct autoobs_runp
{            
    char skdname[80];  // schedule file name
    char run_info_file[80];  // run information file
    char stat_id[3];   // station ID
    char outdir[120];
    char k5outinfo[120];
    char logdir[80];
    char keyword[80];  // key words for 1ch mode (source name)
    int outdirmode;
    int runinfomode;
    char y_or_n[2];
    char freqg[4];
    char freqgtab[4][4];   //={"a","b","c","d"};
    int fixspan;
    int adbit;
    int numch;
    int sfreq;      // sampling frequency code
    char afreq[20]; // sampling frequency ascii
    double dsfreq;  // actual sampling frequency (Hz)
    int naming_type;   // out file naming rule
    int subnet;        // 0: subnet mode OFF   1: subnet mode ON (default)
    char a_subnet[6];
    int f_size_mode;   // 0: no limitation on file size  1: divided into 2GB size (default)
    int nosatkey;         // 1: ignore SATKEY  0: use SATKEY
    int noearly;         // 1: ignore EARLY parameter 0: use EARLY parameter
    listoutdir_t outdirtab;  
    //char keys[10][9];  // support 10 max 8-char satellite keys
} autoobs_runp_t;

typedef struct satkeys    
{            
    int  num;
    char keys[10][9];  // support 10 max 8-char satellite keys
} satkeys_t;

/* set interrupt handler */
void Handler(int n);

void SetHandler(void)
{
    if(signal(SIGINT, Handler) == SIG_ERR){
        printf("Signal handler set up failure!\n");
        abort();
    }
}

void Handler(int n)
{
     char cbuf[80];
     printf("\n\n Interrupt occurred !!\n");
     sprintf(cbuf,"Interrupt occurred !!  AUTOOBS_S OBSERVATION TERMINATED by OPERATOR");
     LogWrite4(&lg,0,0.0,3,cbuf);
     exit(0);
}


void p_parse(char *cbuf, char *para1, char *para2)  //2003-06-18
{
/*
    parse   a = c   to  a (para1)  and c (para2)
*/
    // vex_parse =0;  single statement in a cbuf
    //           =1;  more statement left in a cbuf, start point in cbuf 
    //                is updated 
    char wbuf[10][60];
    int i,ipos,k,kpos,flag,len;

    len=strlen(cbuf);

    //remove extra space in front

    if(cbuf[0]==' '){
        kpos=0;
        for(k=0; k<len; k++){
            if(cbuf[k]!=' '){
                kpos=k;
                break;
            }
        }
        i=0;
        for(k=kpos; k<len; k++){
            cbuf[i]=cbuf[k];
            i++;
        }
        cbuf[i]='\0';
    }
    // len again

    len=strlen(cbuf);


    // parse start

    i=0;
    ipos=0;
    flag=0;
    for(k=0; k<len; k++){
        if(cbuf[k] == ' ' || cbuf[k] == '=' || cbuf[k] == ':'){
            if(flag==1) {  // one parameter end
                wbuf[ipos][i]='\0';
                flag=0;
                i=0;
                ipos++;
                if(ipos > 3) goto exithere;
            }
            if(cbuf[k] != ' '){
                wbuf[ipos][i]=cbuf[k];
                wbuf[ipos][i+1]='\0';
                flag=0;
                i=0;
                ipos++;
                if(ipos > 3) goto exithere;
            }
            goto nextk;
        }
        if(cbuf[k] == ';'){
            kpos=k;
            goto exithere;
        }
        if(cbuf[k] != ' '){
            flag=1;
            wbuf[ipos][i]=cbuf[k];
            i++;
        }
nextk:;
    }
    kpos=k;
exithere:
    strcpy(para1,wbuf[0]);
    strcpy(para2,wbuf[2]);

}

void dir_name_check(char *cbuf)
{
    /* check out dir name accompanied by "/" or not */
    int i;
    i=strlen(cbuf);
    if(cbuf[i-1] != '/') {
       cbuf[i]='/';
       cbuf[i+1]='\0';
    }
}

void get_run_info(char *run_info_file, autoobs_runp_t *runp)
{
       int i,k,j,len;
       char ibuf[100];
       char cbuf[80],para1[20],para2[20];
       char freqgtab[][3]={"a","b","c","d"};
       FILE *stream;
              

       if( (stream  = fopen( run_info_file, "r" )) == NULL ){
          printf("get_run_info: File (%s) not found or Open Error! \n",run_info_file);
          goto exitend2;
       }

  loop1:

       if( fgets( ibuf, 100, stream ) == NULL){
          //printf("File EOF or Read Error!\n");
          goto exitend;
       }
       if(ibuf[0]=='*')  // this is comment line
          goto exit1;
       if(ibuf[0]=='$'){ // this is command line
  cmd_entry:
          if(strncmp(ibuf,"$SKED",5)==0){    // schedule file name get
          sloop1:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 2) goto sloop1;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop1;  // carrige return code check
              if(ibuf[0]=='*') goto sloop1;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", runp->skdname);
              goto loop1;
          }
          
          if(strncmp(ibuf,"$OUT",4)==0){    // out directory info get
             runp->outdirtab.dirnum=0;
          sloop2:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 2) goto sloop2;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop2;  // carrige return code check
              if(ibuf[0]=='*') goto sloop2;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", cbuf);
              dir_name_check(cbuf);
              strcpy(runp->outdirtab.dir[runp->outdirtab.dirnum],cbuf);
              runp->outdirtab.dirnum++;
              goto sloop2;
          }
          
          
          if(strncmp(ibuf,"$SAMPL",6)==0){    // sampling parameters get
          sloop3:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 2) goto sloop3;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop3;  // carrige return code check
              if(ibuf[0]=='*') goto sloop3;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              p_parse(ibuf, para1, para2);
              if(strncmp(para1,"span",4)==0 || strncmp(para1,"fixs",4)==0 ){    // span
                  runp->fixspan=atoi(para2);
              }
              if(strncmp(para1,"sfreq",5)==0){    // sampling frequency
                  runp->sfreq=atoi(para2);
              }
              if(strncmp(para1,"adbit",5)==0){    // adbit
                  runp->adbit=atoi(para2);
              }
              if(strncmp(para1,"numch",5)==0){    // numch
                  runp->numch=atoi(para2);
              }
              goto sloop3;
          }
          
          if(strncmp(ibuf,"$STAT",5)==0){    // station ID  get
          sloop4:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 1) goto sloop4;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop4;  // carrige return code check
              if(ibuf[0]=='*') goto sloop4;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", cbuf);
              strcpy(runp->stat_id,cbuf);
              goto loop1;
          }
          
          
          if(strncmp(ibuf,"$LOG",4)==0){    // Log directory  get
          sloop5:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 1) goto sloop5;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop5;  // carrige return code check
              if(ibuf[0]=='*') goto sloop5;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", cbuf);
              dir_name_check(cbuf);
              strcpy(runp->logdir,cbuf);
              goto loop1;
          }
          
          if(strncmp(ibuf,"$NAMING",7)==0){    // Naming type get
          sloop6:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 1) goto sloop6;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop6;  // carrige return code check
              if(ibuf[0]=='*') goto sloop6;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", cbuf);
              runp->naming_type=atoi(cbuf);
              goto loop1;
          }
          
          if(strncmp(ibuf,"$FREQ_G",7)==0){    // Frequency Group get
          sloop7:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 1) goto sloop7;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' ')) goto sloop7;  // carrige return code check
              if(ibuf[0]=='*') goto sloop7;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", cbuf);
              j=0;
              if(cbuf[0]=='1') j=1;
              if(cbuf[0]=='2') j=2;
              if(cbuf[0]=='3') j=3;
              if(cbuf[0]=='4') j=4;
              if(j!=0){
                  strcpy(runp->freqg,runp->freqgtab[j-1]);
              } else {
                  strcpy(runp->freqg,cbuf);
              }
              goto loop1;
          }
          
          if(strncmp(ibuf,"$SUBNET",7)==0){    // subnet mode get get
          sloop8:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 1) goto sloop8;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop8;  // carrige return code check
              if(ibuf[0]=='*') goto sloop8;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", cbuf);
              if(strncmp(cbuf,"off",3)==0 || strncmp(cbuf,"OFF",3)==0) runp->subnet=0;
              if(strncmp(cbuf,"on",2)==0 || strncmp(cbuf,"ON",2)==0) runp->subnet=1;
              goto loop1;
          }
          
          
          if(strncmp(ibuf,"$FILE_SIZE",10)==0){    // file size limit mode get get
          sloop9:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 1) goto sloop9;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop9;  // carrige return code check
              if(ibuf[0]=='*') goto sloop9;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", cbuf);
              if(strncmp(cbuf,"off",3)==0 || strncmp(cbuf,"OFF",3)==0) runp->f_size_mode=0;
              if(strncmp(cbuf,"on",2)==0 || strncmp(cbuf,"ON",2)==0) runp->f_size_mode=1;
              goto loop1;
          }
          
          
          if(strncmp(ibuf,"$SATKEYS",8)==0){    // satellite key words for 1ch mode
          sloop10:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 1) goto sloop10;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop10;  // carrige return code check
              if(ibuf[0]=='*') goto sloop10;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              len=strlen(ibuf);
              /* remove first extra spaces if existed */ 
              for(i=0; i<len; i++){
                  if(ibuf[i] == ' ') continue;
                  if(ibuf[i] != ' ') {
                      k=i;
                      break;
                  }
              }
              j=0;
              for(i=k; i<len; i++){
                  if(ibuf[i] == '*') break;
                  cbuf[j]=ibuf[i];
                  j++;
              }
              cbuf[j]='\0';
              len=strlen(cbuf);
              /* remove last extra spaces if existed */ 
              for(i=len-1; i >= 0; i--){
                  if(cbuf[i] != ' ') {
                      k=i;
                      break;
                  }
                  if(cbuf[i] == ' ') continue;
              }
              for(i=0; i <= k; i++){
                  runp->keyword[i]=cbuf[i];
              }
              runp->keyword[k+1]='\0';

              //printf("Debug %s\n",keyword);
              goto loop1;
          }
          
          
        }
    exit1:
        if (feof(stream) == 0) 
           goto loop1;         // next line read
  exitend:
        i=fclose( stream ) ;
  exitend2:;
}


void get_outdir(char* k5outinfo, listoutdir_t *outdirtab)
{
       int i;
       char line[100];
       FILE *stream;

       if( (stream  = fopen( k5outinfo, "r" )) == NULL ){
          printf("get_outdir: File (%s) not found or Open Error! \n",k5outinfo);
          goto exitend2;
       }

  loop1:

       if( fgets( line, 100, stream ) == NULL){
          //printf("File EOF or Read Error!\n");
          goto exitend;
       }
       //printf("%s",line);
       if(line[0]=='*') {  
          goto loop1;  // this is comment line (nothing to do)  
       } else {
          /* remove CR code */
          i=strlen(line);
          line[i-1]='\0';
          /* check last character */
          dir_name_check(line);
          //printf("dirnum = %d\n",outdirtab->dirnum);
          strcpy(outdirtab->dir[outdirtab->dirnum],line);
          outdirtab->dirnum++;
       }
       goto loop1;
  exitend:
       i=fclose(stream);
  exitend2:;
}

void parse_sat_keys(char* keywords, satkeys_t *sat)
{
    // keywords should be separated by space and each keyword should be 8 or
    // less characters
    // Max number of key words should be 10
    char abuf[80], cbuf[80];
    char *cset=" ";
    char *p=NULL;
    int k;
   
    strcpy(cbuf,keywords);
    sat->num=0;
    sat->keys[0][0]='\0';
    k=0;
    if (strlen(keywords) <= 0) goto exithere;
    sprintf(abuf,"%s\0",strtok(cbuf, cset));
    strcpy(sat->keys[k], abuf);  // 1st key words
    k++;
    while((p=strtok(NULL,cset))){
        sprintf(abuf,"%s\0",p);
        strcpy(sat->keys[k],abuf);
        k++;
        if(k==10) break;
    }
    sat->num=k;
exithere:;
}

void print_usage1(void)
{
    printf("\n");
    printf("autoobs (%s) Usage : \n",AUTOOBS_VERSION);
    printf("\n");
    printf("   autoobs -runparam_file [-NOSATKEY]\n");
    printf("         where  runparam_file ---- run parameters file name\n");
    printf("                -NOSATKEY     ---- ignore SATKEY\n");
    printf("or\n");
    printf("   autoobs sked_file [station_id [span [sfreq [adbit [numch [outdir\n");
    printf("            [y_or_no [naming_type [freqg [subnet [f_size_mode]]]]]]]]]]]\n");
    printf("            [-NOSATKEY]\n");
    //printf("\n");
    printf("     where  sked_file ---- schedule file name (SKED or VEX type)\n");
    printf("            staion_id ---- station ID (A1 for SKED, A2 for VEX)\n");
    printf("            span ---- duration (sec)\n");
    printf("                      compulsory shorten obs length (dflt 0)\n");
    printf("                      =0 means no effect (observed as scheduled)\n");
    printf("            sfreq ---- sampling frequency\n");
    printf("                       40,100,200,500 (for kHz)\n");
    printf("                       1,2,4,8,16 (for MHz)  (dflt 16)\n");
    printf("            adbit ---- A/D bit  1|2|4|8 (dflt 1)\n");
    printf("            numch ---- number of channel 1|4 (dflt 1)\n");
    printf("            outdir ---- output file directory\n");
    printf("                         -outdir means directory info file\n");
    printf("                      default is current directory\n");
    printf("            y_or_n ---- Obs range limitation reply N|Y\n");
    printf("                      if omitted, u will be asked later\n");
    printf("            naming_type ---- out file naming type selection\n");
    printf("                       1 : XDDDNNNN.dat (default)\n");
    printf("                      -1 : XDDDNNNN.#ch.dat\n");
    printf("                       2 : sidDDDHHMMSSG.dat\n");
    printf("                      -2 : sidDDDHHMMSSG.#ch.dat\n");
    printf("                       3 : expid_sidG_scanid_YYYYDDDDHHMMSS.k5\n");
    printf("            freqg ---- frequency group id used in TypeII file naming\n");
    printf("                      1|2|3|4 or a|b|c|d  (default null)\n");
    printf("            subnet --- Subnet mode control 0: no-subnet, 1: subnet on(dflt)\n");
    printf("            f_size_mode -- 0: no limitation  1: divide into each 2GB size(dflt)\n");
    printf("           -NOSATKEY ---- ignore SATKEY\n");
    printf("           -NOEARLY ---- ignore EARLY start parameter in SKED or VEX\n");
    printf("\n");
}

void print_usage2(void)  // this is a planned interface
{
    printf("\n");
    printf("autoobs (%s) Usage : \n",AUTOOBS_VERSION);
    printf("\n");
    printf("  autoobs [sked_file | -runparam_file] [options]\n");
    printf("\n");
    printf("     where  sked_file ---- schedule file name(SKED type or VEX type) \n");
    printf("            runparam_file ---- run parameters file name\n");
    printf("       options (any order)\n");
    printf("           -run runparam_file ---- run parameters file name\n");
    printf("               (note! if runparam_file is set, below parameters\n");
    printf("                      are ignored except for \"-NOSATKEY\" parameter)\n");
    printf("         === belows are valid in case of no \"-run runparam_file\" ===\n");
    printf("           -sked sked_file ---- schedule file name (either VEX or SKED)\n");
    printf("           -sid staion_id ---- station ID (A1 for SKED, A2 for VEX)\n");
    printf("           -span span ---- duration (sec)\n");
    printf("                      compulsory shorten obs length (dflt 0)\n");
    printf("                      =0 means no effect (observed as scheduled)\n");
    printf("           -sfreq 40|100|200|500|1|2|4|8|16 ---- sampling frequency\n");
    printf("                       40,100,200,500 (for kHz)\n");
    printf("                       1,2,4,8,16 (for MHz)  (dflt 16)\n");
    printf("           -ad 1|2|4|8 ---- A/D bit (dflt 1)\n");
    printf("           -nch 1|4 ---- number of channel (dflt 1)\n");
    printf("           -odir outdir ---- output file directory\n");
    printf("                      default is current directory\n");
    printf("           -dinfo dir_info_file  --- directory info file\n");
    printf("           -nolimit ---- no obs range limitation\n");
    printf("                      if omitted, u will be asked later\n");
    printf("           -naming 1|-1|2|-2|3 ---- out file naming type selection\n");
    printf("                       1 : XDDDNNNN.dat (default)\n");
    printf("                      -1 : XDDDNNNN.#ch.dat\n");
    printf("                       2 : sidDDDHHMMSSG.dat\n");
    printf("                      -2 : sidDDDHHMMSSG.#ch.dat\n");
    printf("                       3 : expid_sidG_scanid_YYYYDDDDHHMMSS.k5\n");
    printf("           -freqg 1|2|3|4(or a|b|c|d) \n");
    printf("                   ---- frequency group id used in TypeII file naming\n");
    printf("                       (default null)\n");
    printf("           -subnet 0|1 --- Subnet mode control\n");
    printf("                           0: no-subnet, 1: subnet on(dflt)\n");
    printf("           -fsize 0|1 -- file size limitation\n");  
    printf("                    0: no limitation  1: divide into each 2GB size(dflt)\n");
    printf("           -NOSATKEY --- ignore any SATKEY\n");
    printf("           -NOEARLY ---- ignore EARLY start parameter in SKED or VEX\n");
    printf("\n");
}

int check_autoobs_command(char *command)
{
    /* check command and parameter type */
    /*   -in-
            command  ---  command to be checked 
         -return value-
            parameter type
            -1: no command (this is not a command)
             0: none
             1: int
             2: double
             3: ascii
    */
    if(strncmp(command,"-run",strlen(command))==0) return 3; 
    if(strncmp(command,"-sked",strlen(command))==0) return 3; 
    if(strncmp(command,"-sid",strlen(command))==0)  return 3; 
    if(strncmp(command,"-span",strlen(command))==0) return 1; 
    if(strncmp(command,"-sfreq",strlen(command))==0) return 1; 
    if(strncmp(command,"-ad",strlen(command))==0) return 1; 
    if(strncmp(command,"-nch",strlen(command))==0) return 1; 
    if(strncmp(command,"-odir",strlen(command))==0) return 3;
    if(strncmp(command,"-dinfo",strlen(command))==0) return 3;
    if(strncmp(command,"-nolimit",strlen(command))==0) return 0;
    if(strncmp(command,"-naming",strlen(command))==0) return 1;
    if(strncmp(command,"-freqg",strlen(command))==0) return 1;
    if(strncmp(command,"-subnet",strlen(command))==0) return 1;
    if(strncmp(command,"-fsize",strlen(command))==0) return 1;
    if(strncmp(command,"-nosatkey",strlen(command))==0) return 0;
    if(strncmp(command,"-noearly",strlen(command))==0) return 0;

    if(strncmp(command,"-RUN",strlen(command))==0) return 3; 
    if(strncmp(command,"-SKED",strlen(command))==0) return 3; 
    if(strncmp(command,"-SID",strlen(command))==0)  return 3; 
    if(strncmp(command,"-SPAN",strlen(command))==0) return 1; 
    if(strncmp(command,"-SFREQ",strlen(command))==0) return 1; 
    if(strncmp(command,"-AD",strlen(command))==0) return 1; 
    if(strncmp(command,"-NCH",strlen(command))==0) return 1; 
    if(strncmp(command,"-ODIR",strlen(command))==0) return 3;
    if(strncmp(command,"-DINFO",strlen(command))==0) return 3;
    if(strncmp(command,"-NOLIMIT",strlen(command))==0) return 0;
    if(strncmp(command,"-NAMING",strlen(command))==0) return 1;
    if(strncmp(command,"-FREQG",strlen(command))==0) return 1;
    if(strncmp(command,"-SUBNET",strlen(command))==0) return 1;
    if(strncmp(command,"-FSIZE",strlen(command))==0) return 1;
    if(strncmp(command,"-NOSATKEY",strlen(command))==0) return 0;
    if(strncmp(command,"-NOEARLY",strlen(command))==0) return 0;
    return -1;
}

void print_autoobs_env(void)
{
    /* print environmental variables */
    printf("   Enviroment variables\n");
    //   K5SKED
    printf("        %s ---   default directory for schedule file\n",K5SKED);
    printf("                      ( %s )",getenv(K5SKED));
    if(getenv(K5SKED)==NULL) {
       printf(",  program deflt is ( %s )\n",K5SKEDDFLT);
    } else {
       printf("\n");
    }
    //   K5OUTINFO
    printf("        %s ---   default out directories information file\n",K5OUTINFO);
    printf("                      ( %s )",getenv(K5OUTINFO));
    if(getenv(K5OUTINFO)==NULL) {
       printf(",  program deflt is ( %s )\n",K5OUTINFODFLT);
    } else {
       printf("\n");
    }
    //   K5LOGDIR
    printf("        %s ---   default log file directory\n",K5LOGDIR);
    printf("                      ( %s )",getenv(K5LOGDIR));
    if(getenv(K5LOGDIR)==NULL) {
       printf(",  program deflt is ( %s )\n",K5LOGDIRDFLT);
    } else {
       printf("\n");
    }
    //   K5NAMING
    printf("        %s ---   default out file naming rule\n",K5NAMING);
    printf("                      old or 1 : Type I, new or 2 : Type II,\n");
    printf("                      -1 : Type -I, -2 : Type -II,\n");
    printf("                      3 : Type III (e-VLBI naming conventions)\n");
    printf("                      ( %s )",getenv(K5NAMING));
    if(getenv(K5NAMING)==NULL) {
       printf(",  program deflt is ( %s )\n",K5NAMINGDFLT);
    } else {
       printf("\n");
    }
    //   K5FREQG
    printf("        %s ---   default frequency group of this PC\n",K5FREQG);
    printf("                    1|2|3|4 or a|b|c|d\n");
    printf("                      ( %s )",getenv(K5FREQG));
    if(getenv(K5FREQG)==NULL) {
       printf(",  program deflt is ( %s )\n",K5FREQGDFLT);
    } else {
       printf("\n");
    }
    //   K5SUBNET
    printf("        %s ---   default subnet mode of this PC\n",K5SUBNET);
    printf("                    on: subnet on, off: subnet off\n");
    printf("                      ( %s )",getenv(K5SUBNET));
    if(getenv(K5SUBNET)==NULL) {
       printf(",  program deflt is ( %s )\n",K5SUBNETDFLT);
    } else {
       printf("\n");
    }
    //   K5SATKEY
    printf("        %s ---   default satellite keyword of 1ch obs of this PC\n",K5SATKEY);
    printf("                      ( %s )",getenv(K5SATKEY));
    if(getenv(K5SATKEY)==NULL) {
       printf(",  program deflt is ( %s )\n",K5SATKEYDFLT);
    } else {
       printf("\n");
    }
    printf("\n");
}

void run_param_get1(int argc, char* argv[], autoobs_runp_t *runp)
/*
   get run parameters
*/
{
    char cbuf[80];
    int i,k,len;

    runp->runinfomode=0;
    if (argc>=2){
       strcpy(runp->skdname,argv[1]);  
       if(runp->skdname[0]=='-'){   // this is k5 run info file name mode
          runp->runinfomode=1;
          len=strlen(runp->skdname);
          /* remove first '-' from skdname */
          for(i=0; i<len-1; i++){
             runp->run_info_file[i]=runp->skdname[i+1];
          }
          runp->run_info_file[len-1]='\0';
       }
    }

    if(runp->runinfomode==1) {
       get_run_info(runp->run_info_file, runp); 
       strcpy(lg.logdir,runp->logdir);
       strcpy(runp->y_or_n,"N");
       strcpy(runp->outdir,runp->outdirtab.dir[0]);
       goto bypass1;
    }
    
    runp->outdirmode=0;
    k=3;
    while(argc>=k){
        //if(strncmp(argv[k-1],"-NOSAT",6)!=0 && 
        //    strncmp(argv[k-1],"-nosat",6)!=0){  // to avoid -NOSATKEY command
        if(strncmp(argv[k-1],"-NOSAT",6)!=0 && 
            strncmp(argv[k-1],"-nosat",6)!=0 &&
            strncmp(argv[k-1],"-NOEARLY",8)!=0 &&
            strncmp(argv[k-1],"-noearly",8)!=0 ){  // to avoid -NOSATKEY -NOEARLY command
            if(k==3){
               len=strlen(argv[2]);
               if(len <= 2){
                  strcpy(runp->stat_id,argv[2]);
               } else {
                  runp->stat_id[0]=argv[2][0];
                  runp->stat_id[1]=argv[2][1];
                  runp->stat_id[2]='\0';
               }
            }
            if (k==4) runp->fixspan = atoi(*(argv + 3 ));
            if (k==5) runp->sfreq = atoi(*(argv + 4 ));
            if (k==6) runp->adbit = atoi(*(argv + 5 ));
            if (k==7) runp->numch = atoi(*(argv + 6 ));

            if (k==8){
               strcpy(runp->outdir,*(argv+7));
               if(runp->outdir[0]=='-'){   // this is k5out info file name mode
                  runp->outdirmode=1;
                  len=strlen(runp->outdir);
                  // remove first '-' from outdir
                  for(i=0; i<len-1; i++){
                    runp->k5outinfo[i]=runp->outdir[i+1];
                  }
                  runp->k5outinfo[len-1]='\0';
               }
            } 
            if (k==9) {
               runp->y_or_n[0]=argv[8][0];
               runp->y_or_n[1]='\0';
            }
            if (k==10) runp->naming_type = atoi(*(argv + 9 ));

            if (k==11){
               strcpy(runp->freqg,*(argv + 10 ));
               i=0;
               if(runp->freqg[0]=='1') i=1;
               if(runp->freqg[0]=='2') i=2;
               if(runp->freqg[0]=='3') i=3;
               if(runp->freqg[0]=='4') i=4;
               if(i!=0){
                  strcpy(runp->freqg,runp->freqgtab[i-1]);
               }
            }
       
            if (k==12) runp->subnet = atoi(*(argv + 11 ));
            if (k==13) runp->f_size_mode = atoi(*(argv + 12 ));
        }
        k++;
    }

    /* check out dir name accompanied by "/" or not */

    if(runp->outdirmode==0){
       dir_name_check(runp->outdir);
    }
       
       
    /* set up out dir table */

    if(runp->outdirmode==0){
       strcpy(runp->outdirtab.dir[0],runp->outdir);
       runp->outdirtab.dirnum=1;
    } else {
       runp->outdirtab.dirnum=0;
    }


    /* get file name that contains 2nd, 3rd, 4th, ... directory candidate for output */

    if(runp->outdirmode==0){
       if(getenv(K5OUTINFO) != NULL){
          strcpy(runp->k5outinfo,getenv(K5OUTINFO));
       } else {
          strcpy(runp->k5outinfo,K5OUTINFODFLT);
       }
    }
       
    get_outdir(runp->k5outinfo, &runp->outdirtab);
    //get_outdir(k5outinfo, &dirnum);

    /* debug */
    //for(i=0; i<runp->outdirtab.dirnum; i++){
    //   printf("%d  %s\n",i,runp->outdirtab.dir[i]);
    //}
 
bypass1:;

    /* subnet ON/OFF ascii character set */
    if(runp->subnet==1){ 
       strcpy(runp->a_subnet,"ON");
    } else {
       strcpy(runp->a_subnet,"OFF");
    }

    /* sampling frequency convert into text for print */
    if(runp->sfreq < 40){
       sprintf(runp->afreq,"%dMHz\0",runp->sfreq);
    }  else {
       sprintf(runp->afreq,"%dkHz\0",runp->sfreq);
    }
    
    /* convert sampling frequency index to real frequency (Hz) */

    if(runp->sfreq < 40){
       runp->dsfreq=(double)runp->sfreq*1.0e6;
    } else {
       runp->dsfreq=(double)runp->sfreq*1.0e3;
    }


    /* check no SATKEY command */
    runp->nosatkey=0;
    if(cmnd_get(argc,argv,"-NOSAT",cbuf)==0) runp->nosatkey=1;
    if(cmnd_get(argc,argv,"-nosat",cbuf)==0) runp->nosatkey=1;

    /* check NOEARLY command */
    runp->noearly=0;
    if(cmnd_get(argc,argv,"-NOEARLY",cbuf)==0) runp->noearly=1;
    if(cmnd_get(argc,argv,"-noearly",cbuf)==0) runp->noearly=1;
    
}

void run_param_get2(int argc, char* argv[], autoobs_runp_t *runp)
/*
   get run parameters for command base interface
*/
{
    char cbuf[80];
    int i,len;

    runp->runinfomode=0;
    runp->outdirmode=0;
    if (argc>=2){
        /* check the 1st character of the first run parameter */
        if(argv[1][0] != '-'){ // this should be a schedule file name
            strcpy(runp->skdname,argv[1]); 
        } else { // this is either run parameter file or command
            /* check whether it is command or not */
            if(check_autoobs_command(argv[1]) < 0) { // this is not a command
               runp->runinfomode=1;
               len=strlen(argv[1]);
               /* remove first '-' */
               for(i=0; i<len-1; i++){
                  runp->run_info_file[i]=argv[1][i+1];
               }
               runp->run_info_file[len-1]='\0';
            }
        }
    }

    if(runp->runinfomode==1) {
       get_run_info(runp->run_info_file, runp); 
       strcpy(lg.logdir,runp->logdir);
       strcpy(runp->y_or_n,"N");
       strcpy(runp->outdir,runp->outdirtab.dir[0]);
       goto bypass1; // goto check NOSATKEY 
    }


    /* all command analysis */

    if(cmnd_get(argc,argv,"-run",cbuf)==0 || cmnd_get(argc,argv,"-RUN",cbuf)==0) {
       runp->runinfomode=1;
       strcpy(runp->run_info_file,cbuf);
       get_run_info(runp->run_info_file, runp); 
       strcpy(lg.logdir,runp->logdir);
       strcpy(runp->y_or_n,"N");
       strcpy(runp->outdir,runp->outdirtab.dir[0]);
       goto bypass1; // goto check NOSATKEY 
    }
    if(cmnd_get(argc,argv,"-sked",cbuf)==0) strcpy(runp->skdname,cbuf);
    if(cmnd_get(argc,argv,"-SKED",cbuf)==0) strcpy(runp->skdname,cbuf);
    if(cmnd_get(argc,argv,"-sid",cbuf)==0) strcpy(runp->stat_id,cbuf);
    if(cmnd_get(argc,argv,"-SID",cbuf)==0) strcpy(runp->stat_id,cbuf);
    if(cmnd_get(argc,argv,"-span",cbuf)==0)  runp->fixspan=atoi(cbuf);
    if(cmnd_get(argc,argv,"-SPAN",cbuf)==0)  runp->fixspan=atoi(cbuf);
    if(cmnd_get(argc,argv,"-sfreq",cbuf)==0)  runp->sfreq=atoi(cbuf);
    if(cmnd_get(argc,argv,"-SFREQ",cbuf)==0)  runp->sfreq=atoi(cbuf);
    if(cmnd_get(argc,argv,"-ad",cbuf)==0)  runp->adbit=atoi(cbuf);
    if(cmnd_get(argc,argv,"-AD",cbuf)==0)  runp->adbit=atoi(cbuf);
    if(cmnd_get(argc,argv,"-nch",cbuf)==0)  runp->numch=atoi(cbuf);
    if(cmnd_get(argc,argv,"-NCH",cbuf)==0)  runp->numch=atoi(cbuf);
    if(cmnd_get(argc,argv,"-odir",cbuf)==0) strcpy(runp->outdir,cbuf);
    if(cmnd_get(argc,argv,"-ODIR",cbuf)==0) strcpy(runp->outdir,cbuf);
    if(cmnd_get(argc,argv,"-dinfo",cbuf)==0) {
        runp->outdirmode=1;
        strcpy(runp->k5outinfo,cbuf);
    }
    if(cmnd_get(argc,argv,"-DINFO",cbuf)==0){
        runp->outdirmode=1;
        strcpy(runp->k5outinfo,cbuf);
    }
    if(cmnd_get(argc,argv,"-nolimit",cbuf)==0) strcpy(runp->y_or_n,"N");
    if(cmnd_get(argc,argv,"-NOLIMIT",cbuf)==0) strcpy(runp->y_or_n,"N");
    if(cmnd_get(argc,argv,"-naming",cbuf)==0)  runp->naming_type=atoi(cbuf);
    if(cmnd_get(argc,argv,"-NAMING",cbuf)==0)  runp->naming_type=atoi(cbuf);
    if(cmnd_get(argc,argv,"-freqg",cbuf)==0 
        || cmnd_get(argc,argv,"-FREQG",cbuf)==0  ) {
        strcpy(runp->freqg,cbuf);
        i=0;
        if(runp->freqg[0]=='1') i=1;
        if(runp->freqg[0]=='2') i=2;
        if(runp->freqg[0]=='3') i=3;
        if(runp->freqg[0]=='4') i=4;
        if(i!=0){
           strcpy(runp->freqg,runp->freqgtab[i-1]);
        }
    }
    if(cmnd_get(argc,argv,"-subnet",cbuf)==0)  runp->subnet=atoi(cbuf);
    if(cmnd_get(argc,argv,"-SUBNET",cbuf)==0)  runp->subnet=atoi(cbuf);
    if(cmnd_get(argc,argv,"-fsize",cbuf)==0)  runp->f_size_mode=atoi(cbuf);
    if(cmnd_get(argc,argv,"-FSIZE",cbuf)==0)  runp->f_size_mode=atoi(cbuf);

    /* check out dir name accompanied by "/" or not */

    if(runp->outdirmode==0){
       dir_name_check(runp->outdir);
    }
       
       
    /* set up out dir table */

    if(runp->outdirmode==0){
       strcpy(runp->outdirtab.dir[0],runp->outdir);
       runp->outdirtab.dirnum=1;
    } else {
       runp->outdirtab.dirnum=0;
    }


    /* get file name that contains 2nd, 3rd, 4th, ... directory candidate for output */

    if(runp->outdirmode==0){
       if(getenv(K5OUTINFO) != NULL){
          strcpy(runp->k5outinfo,getenv(K5OUTINFO));
       } else {
          strcpy(runp->k5outinfo,K5OUTINFODFLT);
       }
    }
       
    get_outdir(runp->k5outinfo, &runp->outdirtab);
    //get_outdir(k5outinfo, &dirnum);

    /* debug */
    //for(i=0; i<runp->outdirtab.dirnum; i++){
    //   printf("%d  %s\n",i,runp->outdirtab.dir[i]);
    //}

bypass1:;

    /* subnet ON/OFF ascii character set */
    if(runp->subnet==1){ 
       strcpy(runp->a_subnet,"ON");
    } else {
       strcpy(runp->a_subnet,"OFF");
    }

    /* sampling frequency convert into text for print */
    if(runp->sfreq < 40){
       sprintf(runp->afreq,"%dMHz\0",runp->sfreq);
    }  else {
       sprintf(runp->afreq,"%dkHz\0",runp->sfreq);
    }
    
    /* convert sampling frequency index to real frequency (Hz) */

    if(runp->sfreq < 40){
       runp->dsfreq=(double)runp->sfreq*1.0e6;
    } else {
       runp->dsfreq=(double)runp->sfreq*1.0e3;
    }

    /* check no SATKEY command */
    runp->nosatkey=0;
    if(cmnd_get(argc,argv,"-nosat",cbuf)==0) runp->nosatkey=1;
    if(cmnd_get(argc,argv,"-NOSAT",cbuf)==0) runp->nosatkey=1;

    /* check NOEARLY command */
    runp->noearly=0;
    if(cmnd_get(argc,argv,"-NOEARLY",cbuf)==0) runp->noearly=1;
    if(cmnd_get(argc,argv,"-noearly",cbuf)==0) runp->noearly=1;
}


void monit_autoobs_runp(autoobs_runp_t runp)
{
       int i;
       printf(" skedname      = %s\n",runp.skdname);    
       printf(" run_info_file = %s\n",runp.run_info_file);
       printf(" station_id    = %s\n",runp.stat_id); 
       printf(" outdir        = %s\n",runp.outdir); 
       printf(" k5outinfo     = %s\n",runp.k5outinfo); 
       printf(" logdir        = %s\n",runp.logdir); 
       printf(" outdirmode    = %d\n",runp.outdirmode); 
       printf(" runinfomode   = %d\n",runp.runinfomode); 
       printf(" fixspan = %d\n",runp.fixspan); 
       printf(" sfreq= %d  afreq= %s\n",runp.sfreq,runp.afreq); 
       printf(" dsfreq= %f\n",runp.dsfreq); 
       printf(" adbit= %d\n",runp.adbit); 
       printf(" numch= %d\n",runp.numch); 
       printf(" naming_type = %d\n",runp.naming_type);
       printf(" subnet = %d  a_subnet = %s\n",runp.subnet,runp.a_subnet);
       printf(" f_size_mode = %d\n",runp.f_size_mode);
       printf(" nosatkey    = %d\n",runp.nosatkey);
       printf(" noearly     = %d\n",runp.noearly);
       printf(" freqgtab : "); 
       for(i=0; i< 4; i++) printf(" %s",runp.freqgtab[i]); 
       printf("\n");
       printf(" freq_g = %s\n",runp.freqg); 
       printf(" y_or_n = %s\n",runp.y_or_n); 
       printf(" satellite keys : %s\n",runp.keyword);
       for(i=0; i< runp.outdirtab.dirnum; i++){
            printf(" outdir candidate :  %s\n",runp.outdirtab.dir[i]); 
       }
}


void init_autoobs_runp(autoobs_runp_t *runp)
{
       /* initialize runp */
    int i;
    char cbuf[60];
    char freqgtab[][4]={"a","b","c","d"};
    char* dflt_dir="./";           // default out file directory (1st)

    runp->skdname[0]='\0';
    runp->stat_id[0]='\0';
    runp->run_info_file[0]='\0';
    runp->outdir[0]='\0';
    runp->k5outinfo[0]='\0';
    runp->logdir[0]='\0';
    runp->keyword[0]='\0';
    //runp->fixspan=10;   // default fix span (sec)
    runp->fixspan=0;   // default fix span (sec)
    runp->sfreq=16;
    runp->adbit=1;
    runp->numch=1;
    runp->afreq[0]='\0';
    runp->dsfreq=0.0;
    runp->outdirmode=0;
    runp->runinfomode=0;

    strcpy(runp->outdir,dflt_dir);

    for(i=0; i<10; i++){
       strcpy(runp->outdirtab.dir[i],"NONE");
    }

    strcpy(runp->outdirtab.dir[0],"./");
    runp->outdirtab.dirnum=1;
    runp->y_or_n[0]='\0';
    runp->f_size_mode=1; // file size limit mode default 

    /* log directory name get from enviroment variable K5LOGDIR if it is set */

    if(runp->logdir[0]=='\0'){
       if(getenv(K5LOGDIR) != NULL){
          strcpy(runp->logdir,getenv(K5LOGDIR));
       } else {
          strcpy(runp->logdir,K5LOGDIRDFLT);
       }
    }
       
    /* naming type set from environment variable */

    if(getenv(K5NAMING) != NULL){
       strcpy(cbuf,getenv(K5NAMING));
    } else {
       strcpy(cbuf,K5NAMINGDFLT);
    }
    runp->naming_type=1; // naming type default (old type)
    if(strncmp(cbuf,"old",3)==0 || strncmp(cbuf,"OLD",3)==0) runp->naming_type=1;
    if(strncmp(cbuf,"new",3)==0 || strncmp(cbuf,"NEW",3)==0) runp->naming_type=2;
    if(strncmp(cbuf,"1",1)==0) runp->naming_type=1;
    if(strncmp(cbuf,"2",1)==0) runp->naming_type=2;
    if(strncmp(cbuf,"3",1)==0) runp->naming_type=3;
       
    /* frequency group set from environment variable */
    if(getenv(K5FREQG) != NULL){
       strcpy(cbuf,getenv(K5FREQG));
    } else {
       strcpy(cbuf,K5FREQGDFLT);
    }

    runp->freqg[0]='\0';  //frequency group default
    i=0;
    if(strncmp(cbuf,"1",1)==0 || strncmp(cbuf,"a",1)==0) i=1;
    if(strncmp(cbuf,"2",1)==0 || strncmp(cbuf,"b",1)==0) i=2;
    if(strncmp(cbuf,"3",1)==0 || strncmp(cbuf,"c",1)==0) i=3;
    if(strncmp(cbuf,"4",1)==0 || strncmp(cbuf,"d",1)==0) i=4;
    if(i != 0) strcpy(runp->freqg,freqgtab[i-1]);

    for(i=0; i<4; i++) strcpy(runp->freqgtab[i],freqgtab[i]); //2005-06-03

    /* subnet mode set from environment variable */

    if(getenv(K5SUBNET) != NULL){
       strcpy(cbuf,getenv(K5SUBNET));
    } else {
       strcpy(cbuf,K5SUBNETDFLT);
    }
    runp->subnet=1; // default (subnet ON)
    if(strncmp(cbuf,"on",2)==0 || strncmp(cbuf,"ON",2)==0) runp->subnet=1;
    if(strncmp(cbuf,"off",3)==0 || strncmp(cbuf,"OFF",3)==0) runp->subnet=0;
    /* get satellite keyword for 1ch obs */
    if(getenv(K5SATKEY) != NULL){
       strcpy(runp->keyword,getenv(K5SATKEY));
    } else {
       strcpy(runp->keyword,K5SATKEYDFLT);
    }
}

int main(int argc, char *argv[])
{
       char dev[] = "/dev/tds0";
       char cw;
       char outfile [120];
       char fname [120];
       char mystation[12];
       char stat_id2[4]; 
       char lstat_id2[4];  //station id in all lower case characters
       char cbuf[160];
       char pcid[4];    // '1','2','3',...
       //char keyword[80];              // key words for 1ch mode (source name)
       char hostname[20];           // host name
       int sindex;                    // station index #
       int ndir;                      //current directory number 
       int rv;
       int i, n, span;
       int nobs, obsspan;  
       int yearPC;  // to keep yaer get from PC
       int iofset;
       int chmode;   // 1 or 4  (1ch mode or 4 ch mode indicator)
       int secold;
       int time_mode;  // time mode from where get
                       //  0: from IP-VLBI board   1: from PC
       float tofset, fsec;
       double d1sec;    // 1sec in day unit  (used for adjust sampling start time 
                        //   to scheduled scan start time)
       double dearlysec;  // parameter to reflect early start time
       double dwork;
       double freeb, freeb2, reqb, reqmb;
       int vexmode;  // 0: normal schedule file    1: VEX file  2005-01-12
       listoutdir_t outdirtab;  
       satkeys_t sat;
       //logval_t lg;  /* now global variables */
       spget_t spg;
       vex_t vex;
       //static sked_data_t skd;
       sked_data_t skd;
       obs_file_naming_t ofn;   
       autoobs_runp_t runp;   // 2005-06-03
       timeval_k5_t ts,te,t1,t2,tnow,tns,tne; 

       if(argc == 1){
          print_usage1();
          printf(" Hit return key for another how to run ");
          //scanf("%s",&cw);  // dummy read
          cw=getchar();  // dummy read to accept just return
          print_usage2();
          printf("\n autoobs env   -- for getting environment variable info\n");
          exit(0);
       }
       if (argc>=2){
           if(strncmp(*(argv+1),"env",3)==0){
               print_autoobs_env();
               printf("\n");
               exit(0);
           }
       }

       /************************/
       /* initialize variables */
       /************************/

       /* initialize runp */
       init_autoobs_runp(&runp);

       /* initialize others */

       d1sec=1.0/86400.0;  // 1 sec in day unit

       //dearlysec=5.0/86400.0; // early start parameter in day unit
       dearlysec=0.0; // early start parameter in day unit (default)

       sat.num=0;
       sat.keys[0][0]='\0';

       lg.hostname[0]='\0';
       lg.logdir[0]='\0';
       lg.logfile[0]='\0';
       lg.logfiletype=LOGFILE_TYPE;  // logfile name type

       /**********************/
       /* get run parameters */
       /**********************/

       if(argc>=2){
          /* if 1st or second parameter is command, run with style 2 */
          if(check_autoobs_command(argv[1]) >= 0 ||
              (argc>=3 && check_autoobs_command(argv[2]) >= 0) ) {
              //printf("param2 was selected!!\n");
              run_param_get2(argc, argv, &runp);
          } else {
              //printf("param1 was selected!!\n");
              run_param_get1(argc, argv, &runp);
          }
       }

       strcpy(lg.logdir,runp.logdir);  // log directory set

       /* check schedule file name (is it acompanied by directory or not) */

       get_true_path(runp.skdname, K5SKED, K5SKEDDFLT,cbuf);
       strcpy(runp.skdname,cbuf);


       /* get host name */

       hostget(hostname);
       strcpy(lg.hostname,hostname);


       time_mode=GET_TIME_FROM_BOARD;  //  current TIME get from IP-VLBI board or not
       //time_mode=0;  //  current TIME get from PC board

       /* parse sat keys for 1 ch observation */

       parse_sat_keys(runp.keyword, &sat);

       if(sat.num==0) strcpy(runp.keyword,"NONE");

       if(runp.nosatkey==1){ //2005-06-04
           sat.num=0;
           strcpy(runp.keyword,"NONE");
       }

       //monit_autoobs_runp(runp);  /* debug */

       spg.adbit=runp.adbit;
       spg.sfreq=runp.sfreq;

       /* schedule file format check original SKED or VEX or OVEX */
       vexmode=0;
       check_sked_type(runp.skdname,skd.skedtype);
       //  returned skd.skedtype is SKED | VEX | OVEX


       if((strncmp(skd.skedtype,"VEX",3)==0) ||
           (strncmp(skd.skedtype,"OVEX",4)==0)) {    /* VEX or OVEX format*/
          vexmode=1;
          strcpy(vex.vex_file_name,runp.skdname);
          vex.neop=0;   /* initialize for no EOP data case*/
          vex_read(&vex);
          //SKED_mon(&vex);    //debug
          //FREQ_mon(&vex);    //debug
          //CLOCK_mon(&vex);  //debug
          //STATION_mon(&vex);  //debug
          vex2sked3(&vex, runp.stat_id, &skd);

       } else {                                   /* SKED format */
          strcpy(skd.skdname,runp.skdname);
          readskd(&skd);
      } 

       if (skd.nobs <= 0) { 
           printf(" No observation scan found in the schedule file %s\n",runp.skdname);
           printf(" Check schedule file!!\n");
           exit(0);
       }

#ifdef EXPCODE_LOW_CASE     // convert to lower-case characters
       for(i=0; i < (int)strlen(skd.expcode); i++){
           if(skd.expcode[i] >= 65 && skd.expcode[i] <= 90) { // this is upper-case char
               skd.expcode[i]+=32;   // convert to lower-case char
           }
       }
#endif


       /* select station if Station is not yet selected */

       if (runp.stat_id[0]=='\0') {
          printf(" ****** STATION ID TABLE *********\n");
          for (i=0; i<skd.nstation ; i++){
              if(vexmode==1){
                  printf("  %s --- %s\n",skd.statid[i],skd.statnam[i]);
              } else {
                  printf("  %s (%2s) --- %s\n",skd.statid[i],skd.statid2[i],skd.statnam[i]);
              }
          }
          printf(" *********************************\n");
          if(vexmode==1){
              printf(" Enter Station ID ----> ");
          } else {
              printf(" Enter 1 character Station ID ----> ");
          }
          scanf("%s",&runp.stat_id);
          printf(" Selected station_id is %s\n",runp.stat_id);

          if(vexmode==1){ // setup sked data again using given stat_id
             vex2sked3(&vex, runp.stat_id, &skd);
          }
       }

       /* Set up "tape early start" parameter */
       if(runp.noearly==0){
          dearlysec=(double)skd.earlysec/86400.0; // early start parameter in day unit
          //printf("EARLY parameter = %d\n",skd.earlysec);
       }

       /* Set Up my station Name */

       mystation[0]='\0';
       stat_id2[0]='\0';
       for (i=0; i<skd.nstation ; i++){
           if (strncmp(skd.statid[i],runp.stat_id,strlen(runp.stat_id))==0){
                //strncpy(mystation,skd.statnam[i],strlen(skd.statnam[i]));
                strcpy(mystation,skd.statnam[i]);
                strcpy(stat_id2,skd.statid2[i]);  // 2char code set 2005-02-03
                printf("  %s (%2s) --- %s\n",runp.stat_id,stat_id2,mystation);
                sindex=i;
                break;
           }
       }

       /* prepare station id in lower case caharcters for Type III naming rule */
       strcpy(lstat_id2,stat_id2);
#ifdef STAT_ID2_LOW_CASE     // convert to lower-case characters
       for(i=0; i < (int)strlen(stat_id2); i++){
           if(stat_id2[i] >= 65 && stat_id2[i] <= 90) { // this is upper-case char
               lstat_id2[i]+=32;   // convert to lower-case char
           }
       }
#endif

       /* prepare pc id in '1','2','3','4' for Type III naming rule */
       pcid[0]='\0'; pcid[1]='\0';
       if(runp.freqg[0] != '\0'){
           strcpy(pcid,runp.freqg);
           if(pcid[0] == 'a') pcid[0]='1';
           if(pcid[0] == 'b') pcid[0]='2';
           if(pcid[0] == 'c') pcid[0]='3';
           if(pcid[0] == 'd') pcid[0]='4';
       }

       /* Pick up scans where given_station ID is included */
       if(vexmode==1) pickup_skd(&skd, runp.stat_id);
          // this must be also done for SKED, but it is omitted to 
          // allow simplified SKED for satellite observation

       /* Craete scan ID in case of running using SKED schedule */
       //    scan id is ddd-hhmm 
       if(vexmode==0) setscanid_skd(&skd);
       //for(i=0; i<skd.nobs; i++){
       //    printf(" %d  %s\n",i,skd.scanid[i]);
       //}

       /* Observation Time Range Check */

       tmval_set(skd.start[0], &ts);  // convert time info to autoobs time_valiables
                                      // ts.year, ts.ddd, ts.hh, ts.mm, ts.ss
                                      // ts.month, ts.mday, ts.second, ts.dtime
       printf("day1st  %d\n",ts.ddd);
       printf("1st  Scan : %04d/%02d/%02d %02d:%02d:%02d (%dsec) %s\n",
                ts.year,ts.month,ts.mday,ts.hh,ts.mm,ts.ss,
                      skd.dura[0],skd.obsstar[0]);

       tmval_set(skd.start[skd.nobs-1], &te);
       printf("Last Scan : %04d/%02d/%02d %02d:%02d:%02d (%dsec) %s\n",
                    te.year,te.month,te.mday,te.hh,te.mm,te.ss,
                      skd.dura[skd.nobs-1],skd.obsstar[skd.nobs-1]);


       /* observation start time save to t1 */

       t1=ts;


       /* all observation finish time (end of last scan) save to t2 */

       obsspan=skd.dura[skd.nobs-1];
       /* subnet consideration */
       if(runp.subnet==1){ // sub-net mode
          // get scan duration of the station
          obsspan=skd.datastop[skd.nobs-1][sindex];
       }
           
       t2=te;
       t2.second+=obsspan;

       tmval_update_second(&t2);  // reflect second (if >=86400) info to actual time variable

       /* Observation range limit if selected so */

       if (runp.y_or_n[0]=='\0') { // interactive mode
          strcpy(cbuf,"\0");
          //printf(" Observation Time range Change? (Y/[N]) ----> ");
          printf(" Observation Time range Change? (Y/N) ----> ");
          scanf("%s",&cbuf);
          if(strncmp(cbuf,"Y",1)==0 || strncmp(cbuf,"y",1)==0 ){
             t1.year=ts.year;
             t2.year=t1.year;
             printf("Enter Start Time (month mday hour min sec)\n");
             scanf("%d %d %d %d %d", &t1.month,&t1.mday,&t1.hh,&t1.mm,&t1.ss);
             tmval_update_md2ddd(&t1); // month, mday -> total day  etc
             printf("Enter Stop  Time (month mday hour min sec)\n");
             scanf("%d %d %d %d %d", &t2.month,&t2.mday,&t2.hh,&t2.mm,&t2.ss);
             tmval_update_md2ddd(&t2);
             printf("dtime1 dtime2 %f %f\n)",t1.dtime,t2.dtime);
          } 
       } else if(strncmp(runp.y_or_n,"Y",1)==0){
            t1.year=ts.year;
            t2.year=t1.year;
            printf("Enter Start Time (month mday hour min sec)\n");
            scanf("%d %d %d %d %d", &t1.month,&t1.mday,&t1.hh,&t1.mm,&t1.ss);
            tmval_update_md2ddd(&t1);
            printf("Enter Stop  Time (month mday hour min sec)\n");
            scanf("%d %d %d %d %d", &t2.month,&t2.mday,&t2.hh,&t2.mm,&t2.ss);
            tmval_update_md2ddd(&t2);
            printf("dtime1 dtime2 %f %f\n)",t1.dtime,t2.dtime);
       }
            

       /* log file name create */
       
       dir_name_check(lg.logdir);

       if(LOGFILE_TYPE == 0){
            sprintf(lg.logfile,"%s%s_%s.%s.log",lg.logdir,skd.expcode,
                runp.stat_id,hostname);
       } else {
            sprintf(lg.logfile,"%s%4d%s.log",lg.logdir,t1.year,hostname);
       }

       /* copy log information to spg */
       spg.lg=lg;       // where lg is gloabal variable, while spg is local variable 

       /* file size mode set tp spgettype variable */
       spg.f_size_mode=runp.f_size_mode;


       /* log start */

       sprintf(cbuf,"*********** AUTOOBS %s START **************",AUTOOBS_VERSION);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf," skedname = %s  (type : %s)",runp.skdname,skd.skedtype);  
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       if(vexmode==0){
       sprintf(cbuf," station_id = %s (%s)",runp.stat_id,stat_id2);LogWrite4(&lg,time_mode,0.0,2,cbuf);
       } else {
       sprintf(cbuf," station_id = %s",runp.stat_id);LogWrite4(&lg,time_mode,0.0,2,cbuf);
       }
       sprintf(cbuf," span = %d",runp.fixspan);      LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf," sfreq= %s",runp.afreq);        LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf," adbit= %d",runp.adbit);        LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf," numch= %d",runp.numch);        LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf," outinfo= %s",runp.k5outinfo);  LogWrite4(&lg,time_mode,0.0,2,cbuf);
       if(runp.subnet == 1){
          sprintf(cbuf," subnetmode =  ON"); 
       } else {
          sprintf(cbuf," subnetmode =  OFF"); 
       }
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       if(runp.naming_type == 1){
          sprintf(cbuf," naming_type= 1    (sidDDDNNNN.dat)"); 
       } else if (runp.naming_type == -1){
          sprintf(cbuf," naming_type= -1    (sidDDDNNNN.#ch.dat)"); 
       } else if (runp.naming_type == 2){
          sprintf(cbuf," naming_type= 2    (sidDDDHHMMSSG.dat)"); 
       } else if (runp.naming_type == -2){
          sprintf(cbuf," naming_type= -2    (sidDDDHHMMSSG.#ch.dat)"); 
       } else if (runp.naming_type == 3){
          sprintf(cbuf," naming_type= 3    (expid_sidG_scanid_YYYYDDDDHHMMSS.k5)"); 
       } else {
          sprintf(cbuf," naming_type= %d",runp.naming_type); 
       }
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       if(runp.freqg[0]=='\0'){
           sprintf(cbuf," freq_group = NULL"); 
       } else {
           sprintf(cbuf," freq_group = %s",runp.freqg); 
       }
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       for(i=0; i< runp.outdirtab.dirnum; i++){
            sprintf(cbuf," outdir candidate :  %s",runp.outdirtab.dir[i]); 
            LogWrite4(&lg,time_mode,0.0,2,cbuf);
       }
       sprintf(cbuf," satellite key words :  %s",runp.keyword); 
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"***************************************************************");
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
    
       SetHandler();   // interrupt handler

       /* check Board Time and System Time, and get offset */

       iofset=0;
       tofset=0.0;

       if(toffset_get(&yearPC, &tofset)<0) exit(0);
        
       ndir=0;

       strcpy(runp.outdir,runp.outdirtab.dir[ndir]);
       ndir++;
       runp.outdirtab.ndir=ndir;   

       /* copy outdir info to spg */ 
       spg.outdirtab.dirnum=runp.outdirtab.dirnum;
       spg.outdirtab.ndir=runp.outdirtab.ndir;
       for(i=0; i< runp.outdirtab.dirnum; i++){
           strcpy(spg.outdirtab.dir[i],outdirtab.dir[i]);
       }

       
       /* log obs range info */

       freeb=GetFreeInfo(runp.outdir);
       if(strncmp(runp.outdirtab.dir[ndir],"NONE",4)==0){
          freeb2=0.0;
       } else {
          freeb2=GetFreeInfo(runp.outdirtab.dir[ndir]);
       }
       sprintf(cbuf,"*****************************************************************");
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"*   AUTOOBS (Automatic observation program for IP-VLBI board)   *");
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"*        (%s by Kashima VLBI Group)         *",AUTOOBS_VERSION);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"*                                                               *");
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       if(time_mode==1){
       sprintf(cbuf,"*         Schedule Time Get from IP-BOARD : Naming Type %-2d      *",
           runp.naming_type);
       } else {
       sprintf(cbuf,"*         Time for scheduling Get from PC : Naming Type %-2d      *",
           runp.naming_type);
       }
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"*****************************************************************");
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Schedule File     :  %s (type : %s)",runp.skdname,skd.skedtype);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Experiment Code   :  %s        Freq_G (%s)",skd.expcode,runp.freqg);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Log File          :  %s",lg.logfile);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Sampling Frequency:  %s,  AD bits: %d,   # of CH: %d",runp.afreq, runp.adbit, runp.numch);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Stations Included :  %s",skd.statnam[0]);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       for (i=1; i<skd.nstation; i++){ 
       sprintf(cbuf,"                    :  %s",skd.statnam[i]); 
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       }
       if(vexmode==0){
       sprintf(cbuf,"  My Station Name   :  %s (My Station ID : %s (%s))   Subnet Mode : %s",
           mystation,runp.stat_id,stat_id2,runp.a_subnet);
       } else {
       sprintf(cbuf,"  My Station Name   :  %s (My Station ID : %s)   Subnet Mode : %s",
           mystation,runp.stat_id,runp.a_subnet);
       }
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Current Out Dir   :  %s    %.1fMB left",runp.outdir,freeb/1.0e6);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Next    Out Dir   :  %s    %.1fMB left",runp.outdirtab.dir[ndir],freeb2/1.0e6);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Total Scan Number :  %d",skd.nobs);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Satellite Keywords:  %s",runp.keyword);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);

       sprintf(cbuf,"  1st  Scan         :  %04d/%02d/%02d %02d:%02d:%02d  (%4dsec)  %s",
                      ts.year,ts.month,ts.mday,ts.hh,ts.mm,ts.ss,skd.dura[0],skd.obsstar[0]);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Last Scan         :  %04d/%02d/%02d %02d:%02d:%02d  (%4dsec)  %s",
                      te.year,te.month,te.mday,te.hh,te.mm,te.ss,skd.dura[skd.nobs-1],skd.obsstar[skd.nobs-1]);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);

       sprintf(cbuf,"  Obs Range (start) :  %04d/%02d/%02d %02d:%02d:%02d",
                      t1.year,t1.month,t1.mday,t1.hh,t1.mm,t1.ss);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Obs Range (end)   :  %04d/%02d/%02d %02d:%02d:%02d",
                      t2.year,t2.month,t2.mday,t2.hh,t2.mm,t2.ss);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       if(runp.noearly==1){                                              //2005-07-25
       sprintf(cbuf,"  EARLY start parameter is ignored by an operator");//2005-07-25
       } else {                                                          //2005-07-25
       sprintf(cbuf,"  EARLY start       :  %d sec",skd.earlysec);       //2005-07-25
       }                                                                 //2005-07-25
       LogWrite4(&lg,time_mode,0.0,2,cbuf);                              //2005-07-25

       /* file naming type variable info set */
       ofn.naming_type=runp.naming_type;
       ofn.tf=ts;
       strcpy(ofn.expcode,skd.expcode);
       strcpy(ofn.statid,runp.stat_id);
       strcpy(ofn.statid2,lstat_id2);
       strcpy(ofn.freqg,runp.freqg);
       strcpy(ofn.pcid,pcid);


        /* Schedule Time Check */
        /* wait till next obs */
       
        nobs=0;

             /*******************************/
 obsloop:    /*   Entry of each scan loop   */
             /*******************************/
       
        if(nobs > skd.nobs) goto exitend;

        /* satellite key word check for setup 1ch mode */

        chmode=runp.numch;
        n=0;
        for(i=0; i<sat.num; i++){
            if(strncmp(skd.obsstar[nobs],sat.keys[i],strlen(sat.keys[i]))==0) n=1; // 1ch obs     addded on 2003-03-13
        }
        if(n==1){ // compulsory set 1ch mode
            chmode=1;
        }


       secold=90000;

       /* check disk free space */

       freeb=GetFreeInfo(runp.outdir);
       if(strncmp(runp.outdirtab.dir[ndir],"NONE",4)==0){
          freeb2=0.0;
       } else {
          freeb2=GetFreeInfo(runp.outdirtab.dir[ndir]);
       }

       /* display observation informations */

       printf("\033[2J");   //  Erase the screen ( move the cursor to home ) for VT100
       printf("\033[H");   //  move the cursor to home
       printf("*****************************************************************\n");
       //printf("*                                                               *\n");
       printf("*    AUTOOBS (%s  by Kashima VLBI Group)    *\n",AUTOOBS_VERSION);
       //printf("*                                                               *\n");
       if(time_mode==1){
       printf("*              Time Get from IP-BOARD : Naming Type %-2d          *\n",
           runp.naming_type);
       } else {
       printf("*              Time Get from PC       : Naming Type %-2d          *\n",
           runp.naming_type);
       }
       printf("*****************************************************************\n");
       printf("\n");

       printf("  Schedule File     :  %s (type : %s)\n",runp.skdname,skd.skedtype);
       printf("  Experiment Code   :  %s          Freq_G (%s)\n",skd.expcode,runp.freqg);
       printf("  Log File          :  %s\n",lg.logfile);
       printf("  Stations Included : ");
       if(skd.nstation < 7){
          for (i=0; i<skd.nstation; i++) printf(" %-8s",skd.statnam[i]);
          printf("\n");
       } else {
          n=6;
          while(1){
             for (i=n-6; i<n; i++) printf(" %-8s",skd.statnam[i]);
             printf("\n");
             n+=6;
             if(n > skd.nstation){
                printf("                      ");
                for (i=n-6; i<skd.nstation; i++) printf(" %-8s",skd.statnam[i]);
                printf("\n");
                break;
             }
             printf("                      ");
          }
       }
       if(vexmode==0){
       printf("  My Station Name   :  %s (My Station ID : %s (%s))   Subnet Mode : %s\n",
           mystation,runp.stat_id,stat_id2,runp.a_subnet);
       } else {
       printf("  My Station Name   :  %s (My Station ID : %s)   Subnet Mode : %s\n",
           mystation,runp.stat_id,runp.a_subnet);
       }
       printf("  Current Out Dir   :  %s    %.1fMB left\n",runp.outdir,freeb/1.0e6);
       printf("  Next    Out Dir   :  %s    %.1fMB left\n",runp.outdirtab.dir[ndir],freeb2/1.0e6);
       printf("  Total Scan Number :  %d       Satellite Mode Keys: %s\n",skd.nobs,runp.keyword);

       printf("  1st  Scan         :  %04d/%02d/%02d %02d:%02d:%02d  (%4dsec)  %s\n",
                      ts.year,ts.month,ts.mday,ts.hh,ts.mm,ts.ss,skd.dura[0],skd.obsstar[0]);
       printf("  Last Scan         :  %04d/%02d/%02d %02d:%02d:%02d  (%4dsec)  %s\n",
                      te.year,te.month,te.mday,te.hh,te.mm,te.ss,skd.dura[skd.nobs-1],skd.obsstar[skd.nobs-1]);

       //printf("  Obs Range (start) :  %04d/%02d/%02d %02d:%02d:%02d\n",
       printf("  Obs Range (start) :  %04d/%02d/%02d %02d:%02d:%02d",   //2005-07-25
                      t1.year,t1.month,t1.mday,t1.hh,t1.mm,t1.ss);

       if(runp.noearly==1){                                             //2005-07-25
          printf("    no EARLY start mode\n");                          //2005-07-25
       } else {                                                         //2005-07-25
          printf("    each scan EARLY start : %d sec\n",skd.earlysec);            //2005-07-25
       }                                                                //2005-07-25

       printf("  Obs Range (end)   :  %04d/%02d/%02d %02d:%02d:%02d\n",
                      t2.year,t2.month,t2.mday,t2.hh,t2.mm,t2.ss);
       printf("  Sampling Condition for Next Scan :  %d-ch %s %d-bit sampling\n",
                      chmode, runp.afreq, runp.adbit);
       printf("\n");

       printf("\033[2B");  // move the cursor 2 lines down

       if(time_mode == 1) {
           if(ip_board_time(&tnow.ddd, &tnow.second, &tnow.hh, &tnow.mm, &tnow.ss)==-1) return -1 ;
           tnow.year=yearPC;
           tmval_update_ddd2md(&tnow); 
       } else {
           getsystimef(&tnow.year, &tnow.month, &tnow.mday, &tnow.hh, &tnow.mm, &tnow.ss, &fsec, &tofset);
           tmval_update_md2ddd(&tnow); 
       }
       printf("  Time Now (UTC)    :  %04d/%02d/%02d %02d:%02d:%02d\n\033[A",
                   tnow.year,tnow.month,tnow.mday,tnow.hh,tnow.mm,tnow.ss);
       printf("\033[2A");  // move the cursor 2 lines up


       while(1){               // observation loop

           /* next observation start time get */

           tmval_set(skd.start[nobs], &tns);

           obsspan=skd.dura[nobs];
           
           /* subnet consideration */

           if(runp.subnet==1){ // sub-net mode
              // get scan duration of this station
              obsspan=skd.datastop[nobs][sindex];
           }
           if(obsspan <= 0) {
              nobs++;
              goto obsloop;  // skip this scan
           }           
           
           /* next observation end time set */

           tne=tns;
           tne.second+=obsspan;
           tmval_update_second(&tne);  // reflect second (if >=86400) info to actual time variable

           //dtimenext2=tns.dtime+(double)obsspan/86400.0;
           
           /* disk capacity check */

           if (runp.fixspan > obsspan || runp.fixspan==0){
              span=obsspan; 
           } else {
              span=runp.fixspan; 
           }

           reqb=runp.dsfreq*(double)runp.adbit*(double)span*(double)chmode; 

           reqb/=8.0;   // in byte unit
           reqmb=reqb/1.0e6;
           freeb=GetFreeInfo(runp.outdir);
           //if(nobs == 1 && ndir==1) freeb=10.0e6;  // for check
           //if(nobs == 3 && ndir==2) freeb=10.0e6;  // for check

//#ifndef TEMP_TEST    // check here

           if(1.5*reqb > freeb){   //  out directory change
               if(spg.outdirtab.ndir < runp.outdirtab.dirnum){ 
                  strcpy(runp.outdir,runp.outdirtab.dir[spg.outdirtab.ndir]); 
                  spg.outdirtab.ndir++;  
                  sprintf(cbuf,"OUT directory changed  :  %s",runp.outdir);
                  LogWrite4(&lg,time_mode,0.0,0,cbuf);
                  ndir=spg.outdirtab.ndir;   // to syncronize 2005-01-30
                  runp.outdirtab.ndir=ndir;       // to syncronize 2005-01-30
                  //strcpy(spg.outdir,outdir); // to syncronize 2005-01-30
                  goto obsloop;
              }
           }

//#endif
           /* out file name creation */

           ofn.obsnum=nobs+1;
           ofn.numch=chmode;
           strcpy(ofn.scanid,skd.scanid[nobs]);
           ofn.tn=tns;
           //ofn.year=tns.year;
           //ofn.ddd=tns.ddd;
           //ofn.hh=tns.hh;
           //ofn.mm=tns.mm;
           //ofn.ss=tns.ss;
           //ofn.naming_type=3;
           scan_file_naming(&ofn, fname);

           strcpy(outfile,runp.outdir);
           strcat(outfile,fname);
           //printf("%s\n",outfile); // debug
           
           /* display next scan info */
           if(runp.fixspan > obsspan || runp.fixspan==0){
              printf("\033[2K  Next Scan(No.%04d):  %04d/%02d/%02d %02d:%02d:%02d  (%dsec)  %s [%.1fMB]\n",
                      nobs+1,tns.year,tns.month,tns.mday,tns.hh,
                      tns.mm,tns.ss,obsspan,skd.obsstar[nobs],reqmb);
                       // \033[2K  means erases the entire current line
              sprintf(cbuf,"OBS# %04d :  %04d/%02d/%02d %02d:%02d:%02d  (%dsec)  %s [%.1fMB]",
                      nobs+1,tns.year,tns.month,tns.mday,tns.hh,
                      tns.mm,tns.ss,obsspan,skd.obsstar[nobs],reqmb);
           } else {
              printf("\033[2K  Next Scan(No.%04d):  %04d/%02d/%02d %02d:%02d:%02d  (%dsec ==>%dsec)  %s [%.1fMB]\n",
                      nobs+1,tns.year,tns.month,tns.mday,tns.hh,
                      tns.mm,tns.ss,obsspan,runp.fixspan,skd.obsstar[nobs], reqmb);
              sprintf(cbuf,"OBS# %04d :  %04d/%02d/%02d %02d:%02d:%02d  (%dsec ==>%dsec)  %s [%.1fMB]",
                      nobs+1,tns.year,tns.month,tns.mday,tns.hh,
                      tns.mm,tns.ss,obsspan,runp.fixspan,skd.obsstar[nobs],reqmb);
           }
           printf("\n");
           LogWrite4(&lg,time_mode,0.0,0,cbuf);

              /**********************************/
    loop1:    /* observation waiting loop entry */
              /**********************************/

           /* compare system time */

#ifndef WIN32
           usleep(SLEEP_USEC);
#else
           tk_sleep(SLEEP_MSEC);
#endif
           if(time_mode == 1) {
              if(ip_board_time(&tnow.ddd, &tnow.second, &tnow.hh, &tnow.mm, &tnow.ss)==-1) return -1 ;
              tnow.year=yearPC;
              tmval_update_ddd2md(&tnow); 
           } else {
              getsystimef(&tnow.year, &tnow.month, &tnow.mday, &tnow.hh, &tnow.mm, &tnow.ss, &fsec, &tofset);
              tmval_update_md2ddd(&tnow); 
           }
           if (t1.dtime > tne.dtime ){// next observation set
              nobs++;
              if(nobs > skd.nobs) goto exitend;
              printf("\033[2A");  // move the cursor 2 lines up
              goto pass2;
           }

           /* check sampling start conditions */

           //if ((tnow.dtime+d1sec) < ts.dtime) goto pass1; // nothing to do
           //if ((tnow.dtime+d1sec) < t1.dtime) goto pass1; // nothing to do
           if ((tnow.dtime+d1sec) < ts.dtime-dearlysec) goto pass1; // nothing to do  2005-07-25
           if ((tnow.dtime+d1sec) < t1.dtime-dearlysec) goto pass1; // nothing to do  2005-07-25
           if (tnow.dtime > t2.dtime) goto exitend;       // no more observation
           if (tnow.dtime > te.dtime) goto exitend;       // no more observation

           //if ((tnow.dtime+d1sec) >= tns.dtime && tnow.dtime < tne.dtime ) {
           if ((tnow.dtime+d1sec) >= tns.dtime-dearlysec && tnow.dtime < tne.dtime ) { // 2005-07-25
              /* sampling start */
              /* obsspan re-calc */
              dwork=(tne.dtime - tnow.dtime-d1sec)*86400.0; 
              obsspan=nint(dwork) ; 
              if (runp.fixspan > obsspan || runp.fixspan==0){
                 span=obsspan; 
              } else {
                 span=runp.fixspan; 
              }
              printf("\033[B");  // move the cursor 1 line down
              printf("\n  OUT file name ==> %s\n",outfile);

              if(runp.numch == 4 && chmode ==1){
                 printf("SATELLITE (1ch) mode!!  1ch sampling now\n");   
              }

              /* sampling parameter set to a spg variable */
              spg.span=span;
              spg.numch=chmode;
              strcpy(spg.outdir,runp.outdir);
              strcpy(spg.outfile,outfile);
              strcpy(spg.fname,fname);

              /* actual sampling start */ 
              rv=sampleget(&spg);
              
              /* sampling finished */
              if(rv > 0) {
                 sprintf(cbuf,"sampler read error occured at least once (%d)",rv);
                 LogWrite4(&lg, time_mode, 0.0, 1, cbuf);
              } 

              /* synchronization outdir info between spg and runp variables */
                      //   outdir possible to be changed in sample get                
              strcpy(runp.outdir,spg.outdir);
              ndir=spg.outdirtab.ndir; 
              runp.outdirtab.ndir=ndir;

              
              if(runp.numch == 4 && chmode ==1){
                 sprintf(cbuf,"SATELLITE (1ch) mode!!  1ch sampled"); 
                 LogWrite4(&lg,time_mode,0.0,2,cbuf);
              }
              sprintf(cbuf,"OBS# %04d : %s %d %d %d %d",nobs+1,outfile,span,runp.sfreq,runp.adbit,chmode);           
              LogWrite4(&lg,time_mode,0.0,1,cbuf);
             
            
              nobs++;

              goto obsloop;
           
           }
           if (tnow.dtime > tne.dtime){ // next observation set
              nobs++;
              printf("\033[2A");  // move the cursor 2 lines up
              goto pass2;
           }

       pass1:
           if (tnow.second != secold){
             printf("  Time Now (UTC)    :  %04d/%02d/%02d %02d:%02d:%02d\n\033[A",
                      tnow.year,tnow.month,tnow.mday,tnow.hh,tnow.mm,tnow.ss);
           }
           secold=tnow.second; 
           goto loop1;
        pass2:;

       }

  exitend:

       printf("\n\n\n"); 
       
       sprintf(cbuf,"*********** AUTOOBS OBSERVATION END **************");
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       
       return 0;
}

