/*********************************************************************/
/*                                                                   */
/*   Program Name :  datachk                                         */
/*                                                                   */
/*      Version:  0.0   2001-08-17                                   */
/*                2.10  2002-06-04                                   */
/*                2.11  2002-10-21  to allow extended file           */
/*                2.20  2004-08-12  add sampling statistics          */
/*                2.21  2005-06-12  minor update                     */
/*                2.30  2005-10-05  achive speed up by factor upto 3 */
/*                2.31  2005-10-07  more faster 100th times          */
/*                2.32  2005-10-11  a bit faster in statistics mode  */
/*                                                                   */
/*                                                                   */
/*      Data Check for PC-VSSP(IP-VLBI) board                        */
/*                                                                   */
/*                                                                   */
/*      Copyright (c) 2002-2005 T.Kondo/NICT All Right Reserved      */
/*                                                                   */
/*********************************************************************/
/*
      usage
             datachk file_name [mode]
                  where   file_name -- K5/VSSP (IP-VLBI) data file name
                          mode     --  1: Display sampling statistics
                                       0: no statistics (default)

        ver 2.11 2002-10-21  support data file created under file extension mode
                             sampling
        ver 2.20 2004-08-12  add display mode that do statistics about sampling biases
        Ver 2.21 2005.06.12  update due to update in common header name
        Ver 2.30 2005.10.05  achieve speed up by factor 1.5 to 3
        Ver 2.31 2005.10.07  more faster by factor 100 or more for bit count
        Ver 2.32 2005.10.11   a bit faster in statistics mode
*/
#define DATACHK_VERSION "Ver. 2005-10-11"

#include "libk5.h"      // common header for IP-VLBI utilities

//#ifndef WIN32
//#include <stdint.h>
//#endif
//#define REWIND_BYTE  0  // define how many bytes rewind if failure in 
                          // header detection  on 1st trial (shoud be multiples of 4)
                          // 0 means rewind till previous header
#define REWIND_BYTE 200 
#define USE_ASM    // use inline assemble function

typedef struct K5VSSP_file     // originated from ip_vlbi_file type
                               // difference is displayed by **K5**
{
    /*
      Note: use of a fpos_t type variable was obsoleted because of
         the compatiblity ploblem of a fpos_t type variable between linux 2.6
         and previous one.
    */
	//unsigned char bit32[4];  // obsoleted in **K5**
	//unsigned char bit64[8]; // obsoleted in **K5**
	unsigned char *ibuf; // will be obsoleted in **K5**
	u_int32_t head32[3]; // **K5**  to contain full header including 
                         // in case of K5/VSSP32
	u_int32_t aux32[10]; // **K5**  to contain aux field 
                         //  size will be adjusted in the future
	u_int32_t *ubuf; // pointer to read buffer **K5**
    int nword;       // ubuf size allocated or to be allocated  **K5**
    int auxbyte;     // # of bytes in aux files  **K5**
                     // valid byte number in aux32
    int *idata;   // pointer to decoded data (integer) container
    double *data;   // pointer to decoded data (R*8) container
	int sfreq;  //  sampling frequency in kHz
	int adbit;  // AD bits (1,2,4,8)
	int numch;  // # of channels
	int icnt;   //  current unit number in a second data 
	int numb;   //  # of bytes in unit # of samples
	int usampl;  // unit # of samples
	int kpos;  // should be obsoleted ?
	int npos;    //  valid number in idata or xdata **K5**
	int imax;    //  # of units in a second data
	int bytesec;    //  # of bytes in a second data  **K5**
	//char* fname; //pointer data file name obsoleted
	char fname[128]; //data file name opened or to be opened  **K5**
	double fsample;  // sampling frequency (Hz)
	double tsample; // sampling period (sec)
	double unitp;   // usampl*tsampl unit length 
    long int seconds; // time (sec) from mid night (00h00m00s)
	int hh, mm, ss;  // time in hour, minutes, seconds 
	int spatn; // detected 2nd sync pattern detected 
	int syncok; // =1 for 32bit all 1 detection // not used
	int format; // =0 for old format , =1 K5/VSSP, =2 K5/VSSP32
                //         =2 is added **K5**
	FILE *stream;
	//fpos_t pos;   // changed to int64_t type 
	//fpos_t size;  // changed to int64_t type
                    // because fpos_t type is no more scalar value in linux 2.6
	int64_t pos;   // current pointer
	int64_t size;  // opened file size in bytes **K5**
	int fh;
	int ch;    
	int ierr;   // error code (general purpose)
	double dtime;  // time at begin of unit
	double dtimest;  // time at begin of file data
	unsigned int bytesread;  // number of bytes read
	int wordread;   // # of 32bit-words read counter (general) **K5**
	int64_t twordread;   // total # of 32bit-word read **K5**
    int vflag, vflag_old;  // byte slip flag  added on 2003-02-14
                           //  if header sync detection is suceed
                           //     set to 1.  else 0
                           //   1: sync detection success 
                           //   0: sync detection error (may be byte slip)
                           //   vflag_old is the previous status
    long int dtime_vflag, dtime_vflag_old; // time at vflag decision made
} K5VSSP_file_t;

int64_t get_file_size(char *fname)
{
    /*  get file size opened */
    struct stat info;
    stat(fname,&info);
    return (int64_t)info.st_size;
    // use %lld to print out whole number that has int64_t type
}

// K5 header check (updated from headerchkn2)
int CheckK5header(K5VSSP_file_t *K5)
/*
    K5 header check only
    K5.head32[2] should be prepared in advance

    return 0 : Header detected
          -1 : K5.head32 is not header
*/
{
    int sdat[16]={1,2,4,8,0,0,0,0,0,0,0,0,0,0,0,0};
    int fdat[16]={40,100,200,500,1000,2000,4000,8000,16000, 
                  32000,64000,128000,256000,512000,1024000,2048000}; 
    int smplbit0, sfreq0, nch;
    u_int32_t syncpat=0x8B000000UL;    // 2nd sync pattern for K5/VSSP
    u_int32_t syncpat32=0x8C000000UL;  // 2nd sync pattern for K5/VSSP32
    int iok;
    iok=-1;

    K5->format=1;
    if((K5->head32[0]==0xFFFFFFFFUL) && ((K5->head32[1] & 0xFF000000UL) == syncpat)){ // this is sync block
        iok=0;
        /* parse sampling information and get time data */ 
        K5->spatn=(int)((K5->head32[1] >> 24) & 0xFF);       // spatn = 139 (0x8B) means sync pattern
        smplbit0 = (K5->head32[1] >> 22) & 0x3;   
        K5->adbit = sdat[smplbit0];      // AD bits
        sfreq0 = ( K5->head32[1] >> 18) & 0xF;
        K5->sfreq=fdat[sfreq0];
        nch = (K5->head32[1] >> 17) & 1;
        if (nch == 1) {
            K5->numch = 4; 
        } else {
            K5->numch = 1;
        }
        K5->seconds=(long int)(K5->head32[1] & 0x1FFFFUL);
        sec2hms(K5->seconds,&K5->hh,&K5->mm,&K5->ss);
    }
    return iok;
}

#ifdef USE_ASM

    #ifdef WIN32
        void decode_data(int *K5pos, int wd, int ilast, int pat, int adbit)  // for VC++ inline assembler
        {
        /*
                eax  --  K5pos
                ebx  --  wd
                ecx (cl)  --  adbit
                edx  --  ilast
                esi  --  loop counter
                edi  --  temporary use of wd
        
                ebp+14h --  pat

        */
                __asm{
                    mov edx, ilast
                    mov eax, K5pos
                    mov ebx, wd
                    mov cl, byte ptr [adbit]
                    //mov ch, byte ptr [pat]  //not necessary
                    xor esi,esi  
            loop_1:
                    mov edi, ebx
                    and edi, pat
                    mov [eax+esi*4], edi
                    shr ebx, cl
                    inc esi
                    cmp esi, edx
                    jnz loop_1
                }
        }
    #else  // for gcc
        void decode_data(int *K5pos, int wd, int ilast, int pat, int adbit)
        {
        /*
                K5pos --  8(%ebp)
                wd    --  12(%ebp)
                ilast --  16(%ebp)
                pat   --  20(%ebp)
                adbit --  24(%ebp)

                eax  --  K5pos
                ebx  --  wd
                ecx (cl)  --  adbit
                edx  --  ilast
                esi  --  loop counter
                edi  --  temporary use of wd
        

        */
            asm volatile (
                 "pushl %eax \n\t"     // save register used in this function
                 "pushl %ebx \n\t"
                 "pushl %ecx \n\t"
                 "pushl %edx \n\t"
                 "pushl %edi \n\t"
                 "pushl %esi \n\t"

                 "movl 16(%ebp), %edx \n\t"
                 "movl  8(%ebp), %eax \n\t"
                 "movl 12(%ebp), %ebx \n\t"

                 "movb 24(%ebp), %cl \n\t"
                 "xor %esi, %esi \n\t"  
            "loop_10: \n\t"
                 "movl %ebx, %edi \n\t"
                 "and  20(%ebp), %edi \n\t"
                 "movl %edi, (%eax,%esi,4) \n\t"
                 "shr %cl, %ebx  \n\t"
                 "inc %esi \n\t"
                 "cmp %edx, %esi \n\t"
                 "jnz loop_10 \n\t"
                 "popl %esi \n\t"    // recover saved register 
                 "popl %edi \n\t"
                 "popl %edx \n\t"
                 "popl %ecx \n\t"
                 "popl %ebx \n\t"
                 "popl %eax \n\t"
                );
        }
    #endif
#else  // normal C function
    void decode_data(int *K5pos, int wd, int ilast, int pat, int adbit)
    {
        int i,ww, wd1;
        wd1=wd;
        for(i=0; i<ilast; i++){
            ww=(int)(wd1 & pat);
            //printf("ww wd1 pat %d %d %d",ww,wd1,pat); //debug
            K5pos[i]=ww;
            wd1>>= adbit ; 
        }
    }
#endif

void parseADdata_3N(K5VSSP_file_t *K5)
{
/*
    decode K5/VSSP (IP-VLBI) data into integer data

      [in]
         K5->ubuf    ---  sampler (K5/VSSP (IP-VLBI) board) out data 
         K5->wordread --  # of data in K5->ubuf (32bit word unit)
         K5->adbit -- sampler A/D resolution bits (1,2,4,8)
      [return]
         K5->idata -- data parsed into each sampled data
         K5->npos  -- valid number of data in xdata array

     Relation between # of ch (numch) and each ch time siries

        sdata(m,k)=xdata(k*numch+m)

             where  numch :  # of channels  (1, 4)
                      m   :  CH#
                      k   :  k-th sampling
        
*/
     int numb8, pat, n, ilast, ipos;
     int pattbl[9]={1,1,3,3,15,15,15,15,255};
     u_int32_t syncpat=0x8B000000UL;    // 2nd sync pattern for K5/VSSP
     u_int32_t wd1,wd2;
     int *K5pos;
     unsigned char bpat;

     if(K5->vflag != 1) goto exitend;
     //numb8=numb;
     numb8=K5->wordread*4;

     ipos=0;
     ilast=32/K5->adbit;
     pat=pattbl[K5->adbit];
     bpat=(unsigned char)pat;
     wd1=K5->ubuf[0];
     //printf("IN PARSE ia %d %d %d %d\n",(u_int32_t)ia[0],(u_int32_t)ia[1],(u_int32_t)ia[2],(u_int32_t)ia[3]);

    K5pos=K5->idata;
     for( n=1; n < K5->wordread; n++){
        wd2=K5->ubuf[n];
        /* check sync pattern */
        if ( (wd1 == 0xFFFFFFFFUL) && ((wd2 & 0xFF000000UL) == syncpat) ) { // sync detected 02-07-17
            //printf("Sync detected \n"); 
            continue;  // pass decoding
        }
//        for(i=0; i<ilast; i++){
//             ww=(int)(wd1 & pat);
//             //printf("ww wd1 pat %d %d %d",ww,wd1,pat); //debug
//             K5->idata[ipos]=ww;
//             wd1=wd1 >> K5->adbit ; 
//             ipos++;
//        }
        /* use of inline assembler */
        K5pos=&K5->idata[ipos];
        decode_data(K5pos, wd1, ilast, pat, K5->adbit);   // use of inline assembler
        //decode_data2(K5pos, wd1, ilast, pat, K5->adbit); // normal C function
                              // no big difference found between decode_data and decode data2
                              //   ex. decode_data  34.279 sec for 8MHz 1bit 4CH 20 sec data
                              //   ex. decode_data2 34.289 sec for 8MHz 1bit 4CH 20 sec data
                              //  direct use like below is also almost same processing speed
                              //     i.e. 34.289 sec
/*
        __asm{
            mov edx, ilast
            mov eax, K5pos
            mov ebx, wd1
            mov cl, bpat
            xor esi,esi  
    loop_1:
            mov edi, ebx
            and edi, pat
            mov [eax+esi*4], edi
            shr ebx, cl
            inc esi
            cmp esi, edx
            jnz loop_1
        }
*/
        ipos+=ilast;

        wd1=wd2;
        //K5pos+=ilast;

     }
/*     
     for(i=0; i<ilast; i++){
         ww=(int)(wd1 & pat);
         //printf("ww wd1 pat %d %d %d",ww,wd1,pat); //debug
         K5->idata[ipos]=ww;
         wd1=wd1 >> K5->adbit ; 
         ipos++;
     }
*/     

     //K5pos=K5->idata+ipos;
     K5pos=&K5->idata[ipos];
     decode_data(K5pos, wd1, ilast, pat, K5->adbit);  
     //decode_data2(K5pos, wd1, ilast, pat, K5->adbit);
     /* use of inline assembler */
/*
     __asm{
        mov edx, ilast
        mov eax, K5pos
        mov ebx, ww
        mov cl, bpat
        xor esi,esi  
loop_2:
        mov edi, ebx
        and edi, pat
        mov [eax+esi*4], edi
        shr ebx, cl
        inc esi
        cmp esi, edx
        jnz loop_2
     }
*/
     ipos+=ilast;

     K5->npos=ipos;

exitend:;
     //printf("numb8,ipos %d %d\n",numb8,ipos);
}


#ifdef USE_ASM
    #ifdef WIN32     // in-line assembler for VC++

    static void hist_count4(int* idata, int npos, int xhalf, int* xhist)
    {
        /*
           cout # of samples that exceeds xhalf 

           idata --- array to contain decoded data
           npos  --- # of samples in idata
           xhalf --  threshold
           xhist --  count results

         count every 4 data step (suppose 4ch data)

         Register assign
           eax  --  pointer to idata
           ebx  --  work (idata value)
           ecx  --  32bit-word counter
           edx  --  npos
           edi  --  plus sign counter
           esi  --  xhalf

        */
        __asm{
            mov edx, npos
            cmp edx, 0
            jz end_func
            mov eax, idata  // start address of decoded data
            mov esi, xhalf
            xor ecx,ecx   // counter for 32bit-word number
            xor edi,edi   // counter for plus sign
    loop_head:
            mov ebx, [eax+ecx*4]
            //mov esi, xhalf
            cmp ebx, esi
            jl pass1
            //inc edi
            add edi, 1    // faster than inc
    pass1:
            add ecx,4  // next  data 
            cmp ecx, edx
            jnz loop_head
    end_func:
            mov eax, xhist
            mov [eax], edi
        }
    } 
    
    static void hist_count1(int* idata, int npos, int xhalf, int* xhist)
    {
        /*
           cout # of samples that exceeds xhalf 

           idata --- array to contain decoded data
           npos  --- # of samples in idata
           xhalf --  threshold
           xhist --  count results

         count every 1 data step (suppose 1ch data)

         Register assign
           eax  --  pointer to idata
           ebx  --  work (idata value)
           ecx  --  32bit-word counter
           edx  --  npos
           edi  --  plus sign counter
           esi  --  xhalf

        */
        __asm{
            mov edx, npos
            cmp edx, 0
            jz end_func
            mov eax, idata  // start address of decoded data
            mov esi, xhalf
            xor ecx,ecx   // counter for 32bit-word number
            xor edi,edi   // counter for plus sign
    loop_head:
            mov ebx, [eax+ecx*4]
            //mov esi, xhalf
            cmp ebx, esi
            jl pass1
            //inc edi
            add edi, 1    // faster than inc
    pass1:
            add ecx,1  // next  data 
            cmp ecx, edx
            jnz loop_head
    end_func:
            mov eax, xhist
            mov [eax], edi
        }
    }
    #else     // gcc inline assemble

    static void hist_count4(int* idata, int npos, int xhalf, int* xhist)
    {
        /*
           idata --- array to contain decoded data   8(%ebp)
           npos  --- # of 32 bits word data in idata 12(%ebp)
           xhist --  array to contain results

           cout # of samples that exceeds xhalf 

           idata --- array to contain decoded data 8(%ebp)
           npos  --- # of samples in idata        12(%ebp)
           xhalf --  threshold                    16(%ebp)
           xhist --  count results                20(%ebp)

         count every 4 data step (suppose 4ch data)

         Register assign
           eax  --  pointer to idata
           ebx  --  work (idata value)
           ecx  --  32bit-word counter
           edx  --  npos
           edi  --  plus sign counter
           esi  --  xhalf

        */
        asm volatile (
            "pushl %eax \n\t"     // save register used in this function
            "pushl %ebx \n\t"
            "pushl %ecx \n\t"
            "pushl %edx \n\t"
            "pushl %edi \n\t"
            "pushl %esi \n\t"
            "movl 12(%ebp), %edx \n\t"
            "cmp $0, %edx  \n\t"
            "jz end_func4 \n\t"
            "movl 8(%ebp), %eax  \n\t" // start address of decoded data
            "movl 16(%ebp), %esi \n\t"
            "xor %ecx, %ecx \n\t"   // counter for 32bit-word number
            "xor %edi, %edi \n\t"   // counter for plus sign
    "loop_head4: \n\t"
            "movl (%eax,%ecx,4), %ebx \n\t"
            "cmp %esi, %ebx  \n\t"
            "jl pass14 \n\t"
            "add $1, %edi \n\t"    // faster than inc
    "pass14: \n\t"
            "add $4, %ecx  \n\t"  // next  data 
            "cmp %edx, %ecx \n\t"
            "jnz loop_head4 \n\t"
    "end_func4: \n\t"
            "movl 20(%ebp), %eax  \n\t"
            "movl %edi, (%eax) \n\t"
            "popl %esi \n\t"    // recover saved register 
            "popl %edi \n\t"
            "popl %edx \n\t"
            "popl %ecx \n\t"
            "popl %ebx \n\t"
            "popl %eax \n\t"
        );
    } 


    static void hist_count1(int* idata, int npos, int xhalf, int* xhist)
    {
        /*
           idata --- array to contain decoded data   8(%ebp)
           npos  --- # of 32 bits word data in idata 12(%ebp)
           xhist --  array to contain results

           cout # of samples that exceeds xhalf 

           idata --- array to contain decoded data 8(%ebp)
           npos  --- # of samples in idata        12(%ebp)
           xhalf --  threshold                    16(%ebp)
           xhist --  count results                20(%ebp)

         count every 1 data step (suppose 1ch data)

         Register assign
           eax  --  pointer to idata
           ebx  --  work (idata value)
           ecx  --  32bit-word counter
           edx  --  npos
           edi  --  plus sign counter
           esi  --  xhalf

        */
        asm volatile (
            "pushl %eax \n\t"     // save register used in this function
            "pushl %ebx \n\t"
            "pushl %ecx \n\t"
            "pushl %edx \n\t"
            "pushl %edi \n\t"
            "pushl %esi \n\t"
            "movl 12(%ebp), %edx \n\t"
            "cmp $0, %edx  \n\t"
            "jz end_func1 \n\t"
            "movl 8(%ebp), %eax  \n\t" // start address of decoded data
            "movl 16(%ebp), %esi \n\t"
            "xor %ecx, %ecx \n\t"   // counter for 32bit-word number
            "xor %edi, %edi \n\t"   // counter for plus sign
    "loop_head1: \n\t"
            "movl (%eax,%ecx,4), %ebx \n\t"
            "cmp %esi, %ebx  \n\t"
            "jl pass11 \n\t"
            "add $1, %edi \n\t"    // faster than inc
    "pass11: \n\t"
            "add $1, %ecx  \n\t"  // next  data 
            "cmp %edx, %ecx \n\t"
            "jnz loop_head1 \n\t"
    "end_func1: \n\t"
            "movl 20(%ebp), %eax  \n\t"
            "movl %edi, (%eax) \n\t"
            "popl %esi \n\t"    // recover saved register 
            "popl %edi \n\t"
            "popl %edx \n\t"
            "popl %ecx \n\t"
            "popl %ebx \n\t"
            "popl %eax \n\t"
        );
    } 

    #endif

#else

    void hist_count4(int* idata, int npos, int xhalf, int* pcount)
    {
        int n,k;
        *pcount=0;
        for(n=0; n<npos; n+=4){
           if(idata[n] >= xhalf) *pcount+=1;
        }
    }

    void hist_count1(int* idata, int npos, int xhalf, int* pcount)
    {
        int n;
        *pcount=0;
        for(n=0; n<npos; n++){
           if(idata[n] >= xhalf) *pcount+=1;
        }
    }

#endif

void hist_count(K5VSSP_file_t *K5, int *xhist)
{
    /*  do count for statistics */
    int xhistw[8];     // 8=4ch*2  (" means sign+  sign- of data)
    int xhalf,n;
    xhalf=1;
    xhalf<<=K5->adbit-1;
    if(K5->numch==4){  // case of 4 channel data
         hist_count4(K5->idata, K5->npos, xhalf, &xhistw[1]);
         hist_count4(K5->idata+1, K5->npos, xhalf, &xhistw[3]);
         hist_count4(K5->idata+2, K5->npos, xhalf, &xhistw[5]);
         hist_count4(K5->idata+3, K5->npos, xhalf, &xhistw[7]);
         xhistw[0]=K5->npos/K5->numch-xhistw[1];
         xhistw[2]=K5->npos/K5->numch-xhistw[3];
         xhistw[4]=K5->npos/K5->numch-xhistw[5];
         xhistw[6]=K5->npos/K5->numch-xhistw[7];
         for(n=0; n<8; n++){
             xhist[n]+=xhistw[n];
         }
    } else {  // case of 1 ch data
         hist_count1(K5->idata, K5->npos, xhalf, &xhistw[1]);
         xhistw[0]=K5->npos-xhistw[1];
         xhist[0]+=xhistw[0];
         xhist[1]+=xhistw[1];
    }
}

void chkfilen2(char* fname)
{
       int i;
       long int numb,npos, secold, tflag, nkai;
       long int bitsec,bitint;
       int rword;
       int f_c;
       int xhist[8];     // 8=4ch*2  (" means sign+  sign- of data)
       int xhalf;
       char cbuf[10], fname2[80];
       int total_frame, bad_frame;
       double dbytes;
       K5VSSP_file_t K5;

       K5.nword=5000;
       K5.ubuf  = (u_int32_t *)malloc(K5.nword*sizeof(u_int32_t)); // K5 data read buffer
       K5.idata  = (int *)malloc(32*K5.nword*sizeof(int)); // K5 data read buffer

       total_frame=0;
       bad_frame=0;

       numb=0;
       npos=0;
       nkai=0;

       tflag=0;
       f_c=0;
       dbytes=0.0;
       for(i=0; i<8; i++) xhist[i]=0; 

       printf("FILE : %s\n",fname);

       if( (K5.stream  = fopen( fname, "rb" )) == NULL ) {
           printf("File Open Error! \n");
           goto exitend2;
       }
       strcpy(K5.fname,fname);
       K5.vflag=0;

       printf("                                                    %% of plus sign data  \n");
       printf(" FMT A/D CH  f(kHz)  TIME   seconds    # of bits    Ch1    Ch2    Ch3    Ch4\n");

       K5.wordread = fread( K5.ubuf, sizeof( u_int32_t ), K5.nword, K5.stream );
       dbytes+=(double)K5.wordread*4.0;
 
       if( ferror( K5.stream ) || K5.wordread==0 ) {
           printf("File EOF or Read Error!\n");
           i=fclose( K5.stream ) ;
           goto exitend2;
       }
       for (i=0; i <2 ; i++){
          K5.head32[i]=K5.ubuf[i];
       }

       rword=K5.wordread;

       while(1){

           if (CheckK5header(&K5) == 0) {  // sync detection
               K5.vflag=1;
               xhalf=(int)(pow(2,K5.adbit-1)+0.5);
               if (numb >= 1 ){
                  total_frame++;
                  //bitint=(numb-8)*8;
                  bitint=(numb-2)*32;
                  bitsec=K5.sfreq*1000*K5.adbit*K5.numch;
                  if(K5.numch==1){
                     printf("%12d %6.1f",bitint,
                      100.0*(double)xhist[1]/(double)(xhist[0]+xhist[1]));
                  } else {
                     printf("%12d %6.1f %6.1f %6.1f %6.1f",bitint,
                      100.0*(double)xhist[1]/(double)(xhist[0]+xhist[1]),
                      100.0*(double)xhist[1*2+1]/(double)(xhist[1*2+0]+xhist[1*2+1]),
                      100.0*(double)xhist[2*2+1]/(double)(xhist[2*2+0]+xhist[2*2+1]),
                      100.0*(double)xhist[3*2+1]/(double)(xhist[3*2+0]+xhist[3*2+1]));
                  } 
                  if (bitint != bitsec){
                      bad_frame++;
                      printf("NG "); 
                      if(tflag ==0){
                         printf("Warning! (# of bits Error)\n"); 
                      } else {
                         printf("Warning! (TIME DISCON and Bits error!)\n");
                      }

                   } else {
                      if(tflag==0){
                         printf("  \n"); 
                      } else {
                         printf(" Warning! (TIME DISCON)\n");
                      }
                   }
               }
               if(nkai >0 && K5.seconds-secold !=1){
                    printf("VSSP %2d  %2d   %4d %02d:%02d:%02d %6dNG", 
                          K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);
                    tflag=1;
               } else {
                    printf("VSSP %2d  %2d   %4d %02d:%02d:%02d %6d  ", 
                          K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);
                    tflag=0;
               }
               for(i=0; i<8; i++) xhist[i]=0; 
               secold=K5.seconds;
               //nkai++;
               numb=0;
           }
           numb++;
           npos++;
           if (npos >= rword){
                npos=0;
                rword=K5.wordread;
           }

           for (i=0; i <1 ; i++){
                K5.head32[i]=K5.head32[i+1];
           }
     
           if (npos+1 == rword){  // next record read
                //printf("Next Record Read!\n"); //debug
                K5.wordread = fread(K5.ubuf, sizeof(u_int32_t), K5.nword, K5.stream);
                parseADdata_3N(&K5);
                dbytes+=(double)K5.wordread*4.0;

                /* sampling balance statistics (only check +/- unbalance) */
                hist_count(&K5, xhist);

                if (K5.wordread ==0 ) {// EOF
                    i=fclose( K5.stream ) ;
                    f_c++;
                    // continuous file name creation     XXXX.datN   where N=1,2,3......
                    strcpy(fname2,fname);
                    sprintf(cbuf,"%d\0",f_c);
                    strcat(fname2,cbuf);
                    /* file open */
                    if( (K5.stream  = fopen( fname2, "rb" )) == NULL ){
                        // file not found
                        goto exitend2;
                    }   
                    printf("\n continuous file (%s) opened!\n",fname2);  
                    printf("                                   ");  
                    strcpy(K5.fname,fname2);
                    K5.wordread = fread( K5.ubuf, sizeof( u_int32_t ), K5.nword, K5.stream );
                    parseADdata_3N(&K5);
                    dbytes+=(double)K5.wordread*4.0;

                    /* sampling balance statistics (only check +/- unbalance) */
                    hist_count(&K5, xhist);

                }
                if( ferror( K5.stream )  ) {
                    printf("File Read Error!\n");
                    break;
                }
           }
           if (npos+1 < rword ){
                K5.head32[1]=K5.ubuf[npos+1];
           } else {
                K5.head32[1]=K5.ubuf[npos+1-rword];
           }
       }
    
// exitend:
        i=fclose( K5.stream ) ;
        printf("\n");
 exitend2:
        free(K5.ubuf);
        free(K5.idata);
        printf("\n");
        printf("%f bytes read\n",dbytes); 
        printf("%d frames out of %d frames (%f%%) are bad frames.\n",
           bad_frame, total_frame, 
           100.0*(double)bad_frame/(double)total_frame);
}


int CheckK5header_with_Read(K5VSSP_file_t *S)
{
    /* 
       Read 64bit Data header check to get sampling parameters 
    
       if first 64bit data is not header block, read file 
       untill finding header block.

       Return
               +ve   # of words-2  read till header finding
               -1    no header found till EOF
    */
      
       int i, i32;
       i32=0;

       S->vflag_old=S->vflag;                  // keep current valid flag
       S->dtime_vflag_old=S->dtime_vflag;  //keep time when vflag decesion made
       S->vflag=0;

       /* first 32bit read */

       S->wordread = fread( S->head32, sizeof( u_int32_t ), 1, S->stream );
       S->twordread+=S->wordread;
       S->pos+=S->wordread*4;  // pointer position is in bytes
       if( ferror( S->stream ) || S->wordread==0 ){ // File EOF or read error
           i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }

       //printf("%X  %X\n",S->head32,S->head32+1);  //debug

loop1:
       /* next 32bit data read */
       S->wordread = fread( S->head32+1, sizeof( u_int32_t ), 1, S->stream );
       S->twordread+=S->wordread;
       S->pos+=S->wordread*4;  // pointer position is in bytes
       if( ferror( S->stream ) || S->wordread==0 ){ // File EOF or read error
	       i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }

       //headerchkn(S->bit64,&iok,&S->spatn,&S->adbit,&S->sfreq,&S->numch,
       //                 &S->seconds,&S->hh,&S->mm,&S->ss);
       if (CheckK5header(S) == 0) {  // sync detection
          S->format=1;          
          goto exit_loop;
       } else {
           S->head32[0]=S->head32[1];
           i32++;
       }
       goto loop1;
exit_loop:
       if(i32 == 0) S->vflag=1;
       S->dtime_vflag=S->seconds;
       return i32;
}

int CheckK5header_with_Read2(K5VSSP_file_t *S, int maxword)
{
    /* 
       Read 64bit Data header check to get sampling parameters 
    
       if first 64bit data is not header block, read file at maximum 
       maxword 32bit-wors untill finding header block.

       Return
               +ve   # of words-2  read till header finding
               -1    no header found after maxword read
    */
      
       int i, i32;
       i32=0;

       S->vflag_old=S->vflag;                  // keep current valid flag
       S->dtime_vflag_old=S->dtime_vflag;  //keep time when vflag decesion made
       S->vflag=0;

       /* first 32bit read */

       S->wordread = fread( S->head32, sizeof( u_int32_t ), 1, S->stream );
       S->twordread+=S->wordread;
       S->pos+=S->wordread*4;  // pointer position is in bytes
       if( ferror( S->stream ) || S->wordread==0 ){ // File EOF or read error
           i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }

       //printf("%X  %X\n",S->head32,S->head32+1);  //debug

loop1:
       /* next 32bit data read */
       S->wordread = fread( S->head32+1, sizeof( u_int32_t ), 1, S->stream );
       S->twordread+=S->wordread;
       S->pos+=S->wordread*4;  // pointer position is in bytes
       if( ferror( S->stream ) || S->wordread==0 ){ // File EOF or read error
	       i=fclose( S->stream ) ;
		   i32=-1;
           goto exit_loop;
	   }

       //headerchkn(S->bit64,&iok,&S->spatn,&S->adbit,&S->sfreq,&S->numch,
       //                 &S->seconds,&S->hh,&S->mm,&S->ss);
       if (CheckK5header(S) == 0) {  // sync detection
          S->format=1;          
          goto exit_loop;
       } else {
           S->head32[0]=S->head32[1];
           i32++;
           if(i32+2 > maxword) return -1;
       }
       goto loop1;
exit_loop:
       if(i32 == 0) S->vflag=1;
       S->dtime_vflag=S->seconds;
       return i32;
}


int fwdNsecK5(int nsec, K5VSSP_file_t *K5)
{
    /* 
       forward fast N sec then check header
       if header doesn't exist, rewind RWIND_BYTE data at first,
       then read data slowly to find out header.
       If still header doesn't exist, rewind 1sec data
       then read data slowly to find out header.

       Note: use of fsetpos() and fgetpos() was obsoleted because of
       the compatiblity ploblem of a fpos_t type variable between linux 2.6
       and previous one

    */
    int k;
    int maxword;
    int nbytes, nback;
    int64_t tword;
	int64_t pos;

    if(nsec < 1) return 0; // nothing to do

    tword=K5->twordread;  // save current word read counter data 
    pos=K5->pos;  // save current position

    k=0;

    nbytes=K5->bytesec*nsec;
    K5->pos+=nbytes;  // pointer position to be set
    if(K5->pos < K5->size){  // not yet reached to EOF
         fseek(K5->stream,nbytes,SEEK_CUR); // move pointer by K5->bytesec
         if( ferror( K5->stream ) ){
            return -1;
         }
    } else {  // exceed EOF so pinter set to EOF
         fseek(K5->stream,(int)(K5->size - pos),SEEK_CUR); // 
         if( ferror( K5->stream ) ){
            return -1;
         }
        K5->wordread=(int)((K5->size - pos)/4); // convert to 32bit-word unit
        K5->twordread+=(int64_t)K5->wordread;
        K5->pos+=K5->wordread*4;  // pointer position is in bytes
        return -1;
    }
    K5->wordread=nbytes/4;
    K5->twordread+=(int64_t)K5->wordread;


    /* read 64 bit header */
    K5->wordread = fread(K5->head32, sizeof(u_int32_t), 2, K5->stream);
    K5->twordread+=K5->wordread;
    K5->pos+=K5->wordread*4;  // pointer position is in bytes
    if( ferror( K5->stream ) || K5->wordread==0 ){ // EOF or File read error
	   return -1;
    }
    if(CheckK5header(&(*K5)) < 0){  // this is not the case of header
       /* rewind file */
       if(REWIND_BYTE == 0){
           fseek(K5->stream,-(int)(K5->pos-pos),SEEK_CUR); // set position at initial position
           K5->twordread=tword;            // word counter retrieve
           K5->pos=pos;  // retrieve initial position
           if((k=CheckK5header_with_Read(&(*K5))) < 0){ // EOF or file read error
               return -1;
           }
       } else {
           maxword=REWIND_BYTE/4;
           fseek(K5->stream,-(int)REWIND_BYTE,SEEK_CUR); // set position at initial position
           K5->twordread-=REWIND_BYTE/4;            // word counter retrieve
           K5->pos-=REWIND_BYTE;  // retrieve initial position
           /* 1st trial */
           if((k=CheckK5header_with_Read2(&(*K5),maxword)) < 0){ // EOF or file read error
               /* 2nd trial */
               if(nsec==1){
                   nback=(int)(K5->pos-pos);  // rewind to initial position
               } else {
                   nback=K5->bytesec;   // rewind just 1 sec
               }

               fseek(K5->stream,-nback,SEEK_CUR); // set position at initial position
               K5->twordread-=nback/4;            // word counter rewind
               K5->pos-=nback;  // initial position rewind
               if((k=CheckK5header_with_Read(&(*K5))) < 0){ // EOF or file read error
                   return -1;
               }
           }
       }
       // get headder after k words read
    }
       
    return k;
}

int fwd1secK5(K5VSSP_file_t *K5)
{
    /* 
       forward fast 1 sec then check header
       if header doesn't exist, rewind RWIND_BYTE data at first,
       then read data slowly to find out header.
       If still header doesn't exist, rewind previous header position
       then read data slowly to find out header.

       Note: use of fsetpos() and fgetpos() was obsoleted because of
       the compatiblity ploblem of a fpos_t type variable between linux 2.6
       and previous one
    */
    int k;
    int maxword;
    int64_t tword;
	int64_t pos;
    tword=K5->twordread;  // save current word read counter data 
    pos=K5->pos;  // save current position

    k=0;

    K5->pos+=K5->bytesec;  // pointer position to be set
    if(K5->pos < K5->size){  // not yet reached to EOF
         fseek(K5->stream,K5->bytesec,SEEK_CUR); // move pointer by K5->bytesec
         if( ferror( K5->stream ) ){
            return -1;
         }
    } else {  // exceed EOF so pinter set to EOF
         fseek(K5->stream,(int)(K5->size - pos),SEEK_CUR); // 
         if( ferror( K5->stream ) ){
            return -1;
         }
        K5->wordread=(int)((K5->size - pos)/4); // convert to 32bit-word unit
        K5->twordread+=(int64_t)K5->wordread;
        K5->pos+=K5->wordread*4;  // pointer position is in bytes
        return -1;
    }
    K5->wordread=K5->bytesec/4;
    K5->twordread+=(int64_t)K5->wordread;

    /* read 64 bit header */
    K5->wordread = fread(K5->head32, sizeof(u_int32_t), 2, K5->stream);
    K5->twordread+=K5->wordread;
    K5->pos+=K5->wordread*4;  // pointer position is in bytes
    if( ferror( K5->stream ) || K5->wordread==0 ){ // EOF or File read error
	   return -1;
    }
    if(CheckK5header(&(*K5)) < 0){  // this is not the case of header
       /* rewind file */
       if(REWIND_BYTE == 0){
           fseek(K5->stream,-(int)(K5->pos-pos),SEEK_CUR); // set position at initial position
           K5->twordread=tword;            // word counter retrieve
           K5->pos=pos;  // retrieve initial position
           if((k=CheckK5header_with_Read(&(*K5))) < 0){ // EOF or file read error
               return -1;
           }
       } else {
           maxword=REWIND_BYTE/4;
           fseek(K5->stream,-(int)REWIND_BYTE,SEEK_CUR); // set position at initial position
           K5->twordread-=REWIND_BYTE/4;            // word counter retrieve
           K5->pos-=REWIND_BYTE;  // retrieve initial position
           /* 1st trial */
           if((k=CheckK5header_with_Read2(&(*K5),maxword)) < 0){ // EOF or file read error
               /* 2nd trial */
               fseek(K5->stream,-(int)(K5->pos-pos),SEEK_CUR); // set position at initial position
               K5->twordread=tword;            // word counter retrieve
               K5->pos=pos;  // retrieve initial position
               if((k=CheckK5header_with_Read(&(*K5))) < 0){ // EOF or file read error
                   return -1;
               }
           }
       }
       // get headder after k words read
    }
       
    return k;
}

void chkfilen(char* fname) 
{
       int usampl=200000;
       int i,kword;
       int64_t tword;
       long int numb,npos, secold, tflag, nkai;
       long int bitsec,bitint;
       int f_c;
       int total_frame, bad_frame;
       char cbuf[10], fname2[80];
       double dbytes;
       K5VSSP_file_t K5;
	   //fpos_t size;

       total_frame=0;
       bad_frame=0;
       numb=0;
       npos=0;
       nkai=0;

       tflag=0;
       f_c=0;
       dbytes=0.0;

       strcpy(K5.fname,fname);



       /* K5/VSSP file open and 1st header read 
               to get sampling information etc */

       K5.size=get_file_size(K5.fname);
       if( (K5.stream  = fopen( fname, "rb" )) == NULL ) {
           printf("File %s Open Error! \n",K5.fname);
           goto exitend2;
       }
       printf("FILE : %s  (%lld bytes)\n",K5.fname,K5.size);

       K5.twordread=0;  // reset word read counter
       K5.pos=0;        // reset current pointer position
       tword=0;

       //if(checkheader(&K5) < 0){
       if((kword=CheckK5header_with_Read(&K5)) < 0){
            printf("File read error! \n");
            //fgetpos(K5.stream,&K5.pos); 
            i=fclose( K5.stream ) ;
            goto exitend2;
       }
       tword+=K5.twordread;
       K5.twordread=0;

       K5.fsample=1000.0*K5.sfreq;   // sampling frequency (Hz)
       K5.tsample=1.0/K5.fsample;            // sampling period (sec)


       bitsec=(long)K5.fsample*K5.adbit*K5.numch;  // # of bits in 1 sec data except header
       K5.bytesec=bitsec/8;                  //# of bytes in 1 sec data
       K5.imax=(K5.sfreq*1000)/usampl;        // # of usampls in a second
       K5.numb=usampl*K5.adbit*K5.numch/8;       // # of bytes in a usampl
       K5.nword=K5.numb/4;                 // # of 32bit-word in a usampl

       K5.unitp=(double)usampl*K5.tsample;   // period in a usampl (sec)

       //K5.ibuf  = (unsigned char *)malloc(K5.numb); // K5 data read buffer
       K5.ubuf  = (u_int32_t *)malloc(K5.nword*sizeof(u_int32_t)); // K5 data read buffer

       if(kword >0){
           printf("First 64bit is not a header block!\n");
       }

       printf(" FMT A/D CH  f(kHz)  TIME   seconds   # of bits\n");
       printf("VSSP %2d  %2d   %4d %02d:%02d:%02d %6d  ", 
                  K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);


loop1:
       //if((kword=fwdNsecK5(1, &K5))<0){
       if((kword=fwd1secK5(&K5))<0){
           tword+=K5.twordread;
           //printf("File EOF  %d\n", K5.twordread);
           goto exit2next;
       }
       //bitint = bitsec+kword*32;
       bitint=(int)(K5.twordread*32-64);  //64bit is headder
       tword+=K5.twordread;
       K5.twordread=0;  // reset read counter
       total_frame++;

       printf("%10d",bitint);
       if (bitint != bitsec){
          bad_frame++;
          printf(" NG "); 
          if(tflag ==0){
              printf("# of bits Error\n"); 
          } else {
              printf("TIME DISCON and Bits error!\n");
          }
       } else {
          if(tflag==0){
              printf("  \n"); 
          } else {
              printf(" TIME DISCON\n");
          }
       }
       printf("VSSP %2d  %2d   %4d %02d:%02d:%02d %6d  ", 
                  K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);
       if(nkai >0 && K5.seconds-secold !=1){
           tflag=1;
       } else {
           tflag=0;
       }
       secold=K5.seconds;
       nkai++;

       goto loop1;

exit2next: // next file candidate read

       i=fclose( K5.stream ) ;
       f_c++;
       // continuous file name creation     XXXX.datN   where N=1,2,3......
       strcpy(fname2,fname);
       sprintf(cbuf,"%d\0",f_c);
       strcat(fname2,cbuf);
       strcpy(K5.fname,fname2);
       K5.size=get_file_size(K5.fname);
       // file open 
       if( (K5.stream  = fopen( K5.fname, "rb" )) == NULL ) {
           //printf("File Open Error! \n");
           goto exitend2;
       }
       K5.pos=0;
       tword-=K5.twordread;   //  to compensate too much count
       //K5.size=get_file_size(K5.stream);

       printf("\n continuous file (%s (%d bytes)) opened!\n",K5.fname,K5.size);  
       printf("                                    ");  
       if((kword=CheckK5header_with_Read(&K5)) < 0){
            tword+=K5.twordread;
            goto exitend;
       }

       bitint=(int)(K5.twordread*32-64);  //64bit is headder
       tword+=K5.twordread;
       K5.twordread=0;  // reset read counter
       total_frame++;
       printf("  %d",bitint);
       if (bitint != bitsec){
          bad_frame++;
          printf("NG "); 
          if(tflag ==0){
              printf("# of bits Error\n"); 
          } else {
              printf("TIME DISCON and Bits error!\n");
          }
       } else {
          if(tflag==0){
              printf("  \n"); 
          } else {
              printf(" TIME DISCON\n");
          }
       }

       printf("VSSP %2d  %2d   %4d %02d:%02d:%02d %6d  ", 
                  K5.adbit,K5.numch,K5.sfreq,K5.hh,K5.mm,K5.ss,K5.seconds);
       if(nkai >0 && K5.seconds-secold !=1){
           tflag=1;
       } else {
           tflag=0;
       }
       secold=K5.seconds;
       goto loop1;
    
 exitend:
       i=fclose( K5.stream ) ;
       //free(K5.ibuf);
       free(K5.ubuf);
       printf("\n");
 exitend2:;
       //tword+=K5.twordread;
       printf("\n");
       printf("%lld bytes read\n",tword*4);
       printf("%d frames out of %d frames (%f%%) are bad frames.\n",
           bad_frame, total_frame, 
           100.0*(double)bad_frame/(double)total_frame);
}

int main(int argc, char *argv[])
{
    char fname[60];
    char* dflt_fname="tds.data";
    int mode;
    double telapsed;           // to measure processing time
    clock_t cstart1sec, cfinish1sec;  // to calculate processing time
    mode=0;
    if(argc < 2) {
       //strcpy(fname,dflt_fname);
       printf("datachk %s\n",DATACHK_VERSION);
       printf("How to run: \n");
       printf("  datachk datafile [mode]\n");
       printf("\n");
       printf("    where  datafile ----- K5 data file name to check\n");
       //printf("                          if 0 is given, internal dflt name will be used\n");
       printf("           mode      ---  display sampling statistics (zero balance)\n");
       printf("                           1: ON   0: no display (default)\n");
       exit(0);
    }
    if (argc>=2){
       strcpy(fname,*(argv+1));
    } 
    if(argc >=3){
       mode = atoi(*(argv + 2 ));
    }
    cstart1sec = clock();
    printf("************ datachk  %s ************\n",DATACHK_VERSION);
    printf("     [Note: # of bits does not include Header]\n");
    if(mode==1){
       chkfilen2(fname);
    } else {
       chkfilen(fname);
    }
    cfinish1sec = clock();
    telapsed = (double)(cfinish1sec - cstart1sec) / CLOCKS_PER_SEC;
    printf("Time elapsed for processing is %f sec\n",telapsed);
    return 0;
}
