/********************************************************************/
/*                                                                  */
/*   Program Name :  monit                                          */
/*                                                                  */
/*      Version:  0.00  2002-05-26                                  */
/*                2.01  2002-10-03                                  */
/*                2.02  2005-06-12   minor change                   */
/*                2.03  2005-09-28   support linux2.6               */
/*                                                                  */
/*      Copyright (c) 2002-2005 T.Kondo/NICT All Right Reserved     */
/*                                                                  */
/********************************************************************/
/*
 ho|ukah̓xj^[vO
        iZbg͕ʂ̃\tgōsj
         by T.Kondo 2002.5.26   
                    2002.10.03   remove 1pps sync 
                    2005.06.12   minor change 
                    2005.09.28   support linux2.6 

    点  monit numch 
         numch  `l 
                       1 (default), 4  

*/

#include "chead.h"      // common header for IP-VLBI utilities

#ifdef linux
  #include <stdint.h>  // necessary for handling uint32_t etc.
  #include <sys/ioctl.h>
  #include <linux/tdsio.h>
#else 
  #include <sys/tdsio.h>  // IP-VLBI board header
#endif
#include "libipvlbi.h"  // IP-VLBIp֐Q(by T.Kondo) 2005-06-12



/* set global variables */
char dev[] = "/dev/tds0";  // IP-VLBI BOARD control variable
int fd_in;                 // IP-VLBI BOARD control variable

/* set interrupt handler */
void Handler(int n);

void SetHandler(void)
{
    if(signal(SIGINT, Handler) == SIG_ERR){
        printf("Signal handler set up failure!\n");
        abort();
    }
}

void Handler(int n)
{
     static int x=0;
     int rv;
     printf("Interrupt occurred !!\n");

     /* sampling stop */

     rv = ioctl(fd_in, TDSIO_SAMPLING_STOP);
     if (rv == -1) {
	 perror("stop sampling failed, anyway stop ");
	 goto end;
     }
     else {
        printf("Stop sampling!\n");
     }

     //close(fd_out);

 end:
     close(fd_in);
     exit(0);
}


/*  f[^W֐
    NۂɃf[^W܂łRbII
    outfile ---- f[^o̓t@C
    span    ---- f[^Wԁ@ibj
    sfreq   ---- TvOg
                 40,100,200,500 (for kHz)
                   1,2,4,8,16 (for MHz)
    adbit   ---- `crbg
                   1,2,4,8
    numch   ---- `l
                    1, 4
*/
int sampleget_monit(char* outfile, int span, int sfreq, int adbit, int numch)
{
	//char outfile[] = "./tds.data";
	//char dev[] = "/dev/tds0";
	//int fd_in;
	int fd_out;
	mode_t ofilemode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
	int rv;
	unsigned int time,day,sec; // foCXǂݍ񂾌݂̎i[
       int i,h,m,s;
       int secforce;
       int ihist[4][8];
       int hmax[4];
       char iline[70];
       char* lev0="          ";
       char* lev1="*         ";
       char* lev2="**        ";
       char* lev3="***       ";
       char* lev4="****      ";
       char* lev5="*****     ";
       char* lev6="******    ";
       char* lev7="*******   ";
       char* lev8="********  ";
       char* lev9="********* ";
       char* lev10="**********";
       char* levd="<         ";
       char* levu="         >";
       int w1,w2,w3,mm;
       //int span, sfreq;
       int headbit=64;  // header bit number  for a new sampler board
       //int headbit=32;  // header bit number  for a old sampler board
       long int allbyte;     // TvOoCg
       long int bitsec;       // Pb̃rbg@iwb_[USrbg܂ށj
       long int bytesec;       // Pb̃oCg@iwb_[USrbg܂ށj
       long int c,k;
       unsigned int sampling1, freq, width, stat;
       int ch, pos;
       u_int8_t sfreq_t, sbit_t, numch_t;
       u_int8_t sampling;
       u_int32_t buf[BUFSIZ * 2];
		//printf("BUFSIZ   %d\n",BUFSIZ);   // BUFSIZ1024łI
       int xdata[BUFSIZ * 16];  // f[^̃f[^[p

	//printf(" span = %d\n",span);    // debug
       //printf(" sfreq= %d\n",sfreq);   // debug

       /* sampling duration check */
       if (span <= 0) {
		 printf(" parameter span (duration in sec) should be positive!\n");
              // nothing to do
              return -1;
       }


       /* sampler open */
	fd_in = open(dev, O_RDONLY);
	//fd_in = open("/dev/tds0", O_RDONLY);
	if (fd_in == -1) {
		perror("device open() error ");
		return -1;
	}

       /* board status check */
 	rv = ioctl(fd_in, TDSIO_GET_STATUS, &stat);
	if (rv == -1) {
		perror("ioctl() status get failed ");
		goto end;
	}
	else {
              /* satatus get again just in case */
 	       rv = ioctl(fd_in, TDSIO_GET_STATUS, &stat);
	       if (rv == -1) {
		    perror("ioctl() status get failed ");
		    goto end;
	       }
	       //printf("ioctl() status get succeed\n");
             // printf(" status = %X\n",stat);
              if((stat & 1) != 1){
                  printf("Warning! 10MHz signal is not supplied. \n");
                  printf("Check 10MHz signal and try again!\n");
                  goto end;
              }
              if((stat & 2) != 2){
                  printf("Warning! 1PPS signal is not supplied. \n");
                  printf("Anyway, continue the process!\n");
              }
              //if((stat & 3) != 3){
              //    printf("Check above signal connection and try again!\n");
              //    goto end;
              //}
	}
      
       /* output file open */
/*
	fd_out = open(outfile, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
	if (fd_out == -1) {
		perror("output file open() error");
		goto end;
	}
*/
       /* sampling frequency set */

       if      (sfreq ==  40)  { sfreq_t =  TDS_SAMPLING_40KHZ; }
       else if (sfreq == 100)  { sfreq_t =  TDS_SAMPLING_100KHZ; }
       else if (sfreq == 200)  { sfreq_t =  TDS_SAMPLING_200KHZ; }
       else if (sfreq == 500)  { sfreq_t =  TDS_SAMPLING_500KHZ; }
       else if (sfreq ==   1)  { sfreq_t =  TDS_SAMPLING_1MHZ; }
       else if (sfreq ==   2)  { sfreq_t =  TDS_SAMPLING_2MHZ; }
       else if (sfreq ==   4)  { sfreq_t =  TDS_SAMPLING_4MHZ; }
       else if (sfreq ==   8)  { sfreq_t =  TDS_SAMPLING_8MHZ; }
       else if (sfreq ==  16)  { sfreq_t =  TDS_SAMPLING_16MHZ; }
       else { 
		printf(" parameter sfreq  (sampling frequency) must be either one of followings:\n");
		printf("              40,100,200,500 (for kHz)\n");
		printf("              1,2,4,8,16 (for MHz)\n");
              return -1;
       }
      
       /* AD resolution set */
       if      (adbit ==  1)  { sbit_t =  TDS_SAMPLING_1BIT; }
       else if (adbit ==  2)  { sbit_t =  TDS_SAMPLING_2BIT; }
       else if (adbit ==  4)  { sbit_t =  TDS_SAMPLING_4BIT; }
       else if (adbit ==  8)  { sbit_t =  TDS_SAMPLING_8BIT; }
       else { 
		printf(" parameter adbit (A/D resolution) must be either one of followings:\n");
		printf("              1,2,4,8\n");
              return -1;
       }

       /* number of channels */
       if      (numch ==  1)  { numch_t =  TDS_SAMPLING_1CH; }
       else if (numch ==  4)  { numch_t =  TDS_SAMPLING_4CH; }
       else { 
		printf(" parameter numch (number of channels) must be either one of followings:\n");
		printf("              1,4\n");
              return -1;
       }

       /* sampling parameters set */
       sampling =  sfreq_t | sbit_t | numch_t;

       if (sfreq >= 40) { bitsec=sfreq*1000 + headbit ;}
       else { bitsec=sfreq*1000*1000 + headbit ;}

       bitsec=bitsec*adbit*numch;   // bit number per second

       bytesec=bitsec/8;
       allbyte=bytesec*span;
       if (allbyte > 2000000000)
       {
          allbyte= 2000000000;
          secforce=allbyte/bytesec;
          printf(" # of bytes force to set to 2GB because it exceeds 2GB\n");  // debug
          printf(" Obserbasation span (sec) reduced to %d\n",secforce);  // debug
        }

       printf(" bitsec  = %d\n",bitsec);  // debug
       printf(" bytesec = %d\n",bytesec);  // debug
       printf(" allbyte = %d\n",allbyte);  // debug

	//sampling = TDS_SAMPLING_4MHZ | TDS_SAMPLING_1BIT | TDS_SAMPLING_1CH;
	rv = ioctl(fd_in, TDSIO_SET_SAMPLING, &sampling);
	if (rv == -1) {
		perror("ioctl() sampling set failed ");
		goto end;
	}
	else {
		printf("ioctl() sampling set succeed\n");
	}

       /* get sampling parameters from a board */
	rv = ioctl(fd_in,TDSIO_GET_SAMPLING,&sampling1);
	if (rv == -1) {
		perror("ioctl() get sampling parameter failed ");
		goto end;
	}
	else {
		printf("ioctl() get sampling parameter succeed\n");
		freq = TDS_GET_SAMP_FREQ(sampling1);
		width = TDS_GET_SAMP_WIDTH(sampling1);
		ch = TDS_GET_SAMP_CH(sampling1);
		if(freq == TDS_SAMPLING_40KHZ)
			freq = 40;
		if(freq == TDS_SAMPLING_100KHZ)
			freq = 100;
		if(freq == TDS_SAMPLING_200KHZ)
			freq = 200;
		if(freq == TDS_SAMPLING_500KHZ)
			freq = 500;
		if(freq == TDS_SAMPLING_1MHZ)
			freq = 1000;
		if(freq == TDS_SAMPLING_2MHZ)
			freq = 2000;
		if(freq == TDS_SAMPLING_4MHZ)
			freq = 4000;
		if(freq == TDS_SAMPLING_8MHZ)
			freq = 8000;
		if(freq == TDS_SAMPLING_16MHZ)
			freq = 16000;
		if(width == TDS_SAMPLING_1BIT)
			width = 1;
		if(width == TDS_SAMPLING_2BIT)
			width = 2;
		if(width == TDS_SAMPLING_4BIT)
			width = 4;
		if(width == TDS_SAMPLING_8BIT)
			width = 8;
		if(ch == TDS_SAMPLING_4CH)
			ch = 4;
		else
			ch =1;
		printf("sampling setting is %d kHz %d BIT %d CH\n",freq,width,ch);
	}	

       /* board time get */
 /*
	rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
	if (rv == -1) {
		perror("ioctl() time get failed ");
		return -1;
	}
       day=TDS_GET_DAYS(time);
       sec=TDS_GET_SEC(time);
       sec2hms(sec,&h,&m,&s);
	//printf("get count day from device [%5d]\n",TDS_GET_DAYS(time));
	//printf("get count sec from device [%5d]\n",TDS_GET_SEC(time));
 	printf("%03d %02d:%02d:%02d\n",day,h,m,s);
 */

       
	rv = ioctl(fd_in, TDSIO_BUFFER_CLEAR);   // {[hFIFÕNA[
	if (rv == -1) {
		perror("ioctl() FIFO clear failed ");
		goto end;
	}
	else {
		printf("ioctl() FIFO clear succeed\n");
	}
/* 
	rv = ioctl(fd_in, TDSIO_SYNC_1PPS);           // Poorւ̓ obsoleted 2002-10-3
	if (rv == -1) {                               // 
		perror("ioctl() 1pps sync failed ");
		goto end;
	}
	else {
		printf("ioctl() 1pps sync succeed\n");
	}
*/

/*	rv = ioctl(fd_in, TDSIO_SYNC_TIME);            // O͎͂gpȂ
	if (rv == -1) {
		perror("ioctl() time sync failed ");
		goto end;
	}
	else {
		printf("ioctl() time sync succeed\n");
	}
*/
	rv = ioctl(fd_in, TDSIO_SAMPLING_START);
	if (rv == -1) {
		perror("ioctl() start sampling failed ");
              printf("Probably time was not set for the IP-VLBI board.\n");
              printf("Please set time using timesettk and try again.\n");
		goto end;
	}
	else {
		printf("ioctl() start sampling succeed\n");
	}

  /* MONITOR START */
       printf("\033[2J");   //  Erase the screen ( move the cursor to home ) for VT100
       printf("\033[H");   //  move the cursor to home
       printf("********************************************************************\n");
       printf("*                                                                  *\n");
       printf("*     MONIT (signal level monitor through K5/VSSP(IP-VLBI) board)  *\n");
       printf("*                     (Ver 2.03  2005-09-28)                       *\n");
       printf("*                             by                                   *\n");
       printf("*                            NICT                                  *\n");
       printf("*                                                                  *\n");
       printf("*                        CTRL C  for STOP                          *\n");
       printf("*                                                                  *\n");
       printf("*******T***********************************************KONDO********\n");
       printf("\n");
	{
             if(numch == 1)
             {
              printf("              CH 1\n");
              } else {
              printf("              CH 1         CH 2         CH 3         CH 4\n");
              }
		c = 0;

              /* histgram data initialize */
              for(k=0; k<8; k++)
              {
                  for(w1=0; w1<4; w1++)
                  {
                      ihist[w1][k]=0;
                  }
              }
             
		//while (c < BUFSIZ * BUFSIZ) {
		while (c <=  allbyte) {
			int nbyte = sizeof(buf);   // nbyte = 8192

			rv = read(fd_in, buf, nbyte);
			if (rv == -1) {
				perror("read() failed ");
				goto end;
			}

          //           if((c != 1) && (c !=0)){ goto bypass1;}

                     parseADdata(buf, rv, adbit, xdata, &pos);
                    /* Histgram Statistics */

                     w3=0;
                     for(k=0; k<pos; k++)
                     {
                       w1=xdata[k];
                       if(adbit == 8) {w2=w1/32; }
                       if(adbit == 4) {w2=w1/2; }
                       if(adbit == 2) {w2=w1; }
                       if(adbit == 1) {w2=w1; }
                       ihist[w3][w2]+=1;
                       
                       w3++;
                       if( w3 == 4 ) { w3 =0; }
                       if( numch == 1 ) { w3 =0; }
                     }

                     if((c != 1) && (c !=0)){ goto bypass1;}
                     
                     /* Maximum found by Channel */
                     for(mm=0; mm<numch; mm++)
                     {
                         w3=0;
                         for(k=0; k<8; k++)
                         {
                            if(ihist[mm][k] > w3){ w3=ihist[mm][k]; }
                         }
                         hmax[mm]=w3;
                     }
                    // printf("debug4\n");  // debug
                   //  printf(" MAX %d %d %d %d\n",hmax[0],hmax[1],hmax[2],hmax[3]);

                     if(c !=0 ) {
                        printf("\033[9A");  // J[\ɂXs
                     }


                     /* Histgram Plot by Level */
                      for(k=7; k>=0; k--)
                      {
                         if(k==7 || k==0)
                         {  
                             if(k==7) {strcpy(iline, "  + FULL ");}
                             if(k==0) {strcpy(iline, "  - FULL ");}
                          } else {
                              strcpy(iline, "         ");
                          }
                         for(mm=0; mm<numch; mm++)
                         {
                             strcat(iline, "  |");
                             w3=hmax[mm];
                             if(w3 < 10){ w3=10;}
                             w2=w3/10;
                             
                             w1=ihist[mm][k];
                         //    printf(" k = %d  ch = %d  W3 = %d  w1= %d\n",k,mm,w3,w1);
                             w1=w1/w2;
                        
                             if((w1 >= 0) && (w1 <= 10))
                             {
                             if(w1 == 0 ){ strcat(iline,lev0);}
                             if(w1 == 1 ){ strcat(iline,lev1);}
                             if(w1 == 2 ){ strcat(iline,lev2);}
                             if(w1 == 3 ){ strcat(iline,lev3);}
                             if(w1 == 4 ){ strcat(iline,lev4);}
                             if(w1 == 5 ){ strcat(iline,lev5);}
                             if(w1 == 6 ){ strcat(iline,lev6);}
                             if(w1 == 7 ){ strcat(iline,lev7);}
                             if(w1 == 8 ){ strcat(iline,lev8);}
                             if(w1 == 9 ){ strcat(iline,lev9);}
                             if(w1 == 10 ){ strcat(iline,lev10);}
                             }
                             else
                             {
                                 strcat(iline,lev0);
                             }
                          
                         }
                         printf(" %s\n",iline);
                       //  printf(" k = %d  %d  %d %d %d\n",k,ihist[0][k],ihist[1][k],
                       //                 ihist[2][k],ihist[3][k]);
                         
                     }
                     printf("\n");

                      /* histgram data initialize */
                     for(k=0; k<8; k++)
                     {
                         for(w1=0; w1<4; w1++)
                         {
                               ihist[w1][k]=0;
                          }
                      }
                     


                     //printf("\033[9A");  // J[\ɂXs
                    bypass1:  ;
                     //printf(" nbyte = %d\n",nbyte);   // debug
			// rv = write(fd_out, buf, rv);
			//c= c+ nbyte;
                    c++;
                    if(c == 40*numch ){c=1;}
		}
	}

	rv = ioctl(fd_in, TDSIO_SAMPLING_STOP);
	if (rv == -1) {
		perror("ioctl() stop sampling failed ");
		goto end;
	}
	else {
		printf("ioctl() stop sampling succeed\n");
	}
/*
	close(fd_out);
*/
       /* board time get */
/*
	rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
	if (rv == -1) {
		perror("ioctl() time get failed ");
		return -1;
	}
       day=TDS_GET_DAYS(time);
       sec=TDS_GET_SEC(time);
       sec2hms(sec,&h,&m,&s);
	//printf("get count day from device [%5d]\n",TDS_GET_DAYS(time));
	//printf("get count sec from device [%5d]\n",TDS_GET_SEC(time));
 	printf("%03d %02d:%02d:%02d\n",day,h,m,s);
*/

 end:
	close(fd_in);

	return rv;
}


int main(int argc, char **argv)
{
	//char dev[] = "/dev/tds0";
	char outfile[] = "./tds.data";
	int rv;
       int span, sfreq, adbit, numch;

	if(argc != 2){
		printf("Usage: monit numch\n");
		printf("         where  numch ---- number of channel used\n");
		printf("                           1,4\n");
		exit(0);
	}

       
	numch = atoi(*(argv + 1 ));
       if(numch != 1) {numch=4;}
       span=100;
       sfreq=1;
       adbit = 8;
/*
	printf(" span = %d\n",span);    // debug
       printf(" sfreq= %d\n",sfreq);   // debug
       printf(" adbit= %d\n",adbit);   // debug
       printf(" numch= %d\n",numch);   // debug
*/
       printf("*******************************************\n");
       printf("*  K5/VSSP (IP-VLBI) board signal monitor *\n");
       printf("*       MONIT (Ver 2.03 2005/09/28)       *\n");
       printf("*                by                       *\n");
       printf("*               NICT                      *\n");
       printf("*******************************************\n");
       printf(" \n");

       SetHandler();
       rv=sampleget_monit(outfile, span, sfreq, adbit, numch);


       return 0;
}
