/********************************************************************/
/*                                                                  */
/*   Program Name :  sampling                                       */
/*                                                                  */
/*      Version:  0.00  2001-08-17                                  */
/*                1.00  2002-05-26                                  */
/*                1.10  2004-05-23                                  */
/*                1.11  2005-06-12  minor update                    */
/*                1.12  2005-06-16  Alow big file by command        */
/*                1.13  2005-09-28   support linux2.6               */
/*                                                                  */
/*      Copyright (c) 2002-2005 T.Kondo/NICT All Right Reserved     */
/*                                                                  */
/********************************************************************/
/*

 ho|ukah ItCTvOvO
        iZbg͕ʂ̃\tgōs)
         by T.Kondo    2001.8.17
                1.12   2005.6.16 Alow big file by command
                1.13   2005-09-28   support linux2.6               

    点 sampling span sfreq adbit numch [filename [logfile]]
          span  Wԁibj
                      ̒l2GByteȏP̃t@CƂč쐬
                      ̒lł2Gbyteɕt@C쐬
                sfreq  TvOg
                       40,100,200,500  (kHz)
                       1,2,4,8,16      (MHz)
                adbit  `c\irbg)
                       1,2,4,8 
                numch  `l
                       1 ܂ 4
                filename -- f[^t@C
                      ftHgtds.data
                logfile -- Ot@C
                      ftHgsampling.log
                      ftHg̃Ot@C͏㏑
                      [U[logfilep[^Ŏw肵Ot@Cɂ
                      O̓Ayh
*/
#define SAMPLING_VERSION "Ver. 2005-09-28"

#include "chead.h"      // common header for IP-VLBI utilities

#include "libipvlbi_board.h"  // IP-VLBITv[{[hp֐Q(by T.Kondo)



int main(int argc, char **argv)
{
    char outfile[60];
    char cbuf[100];
    char* dflt_fname="tds.data";
    char* dflt_logfile="sampling.log";
    char logfile[80];
    int rv;
    int span, sfreq, adbit, numch;
    int logmode;
    spget_t spg;
    logval_t lg;

    if(argc != 5 && argc !=6 && argc !=7){
          printf("Sampling %s\n",SAMPLING_VERSION);
          printf("Usage: sampling span sfreq adbit numch [filename [logfile]]\n");
          printf("\n");
          printf("         where  span ---- duration (sec)\n");
          printf("                          (-VE means allow big file)\n");
          printf("                          (otherwise devided into 2GB files)\n");
          printf("                sfreq ---- sampling frequency\n");
          printf("                           40,100,200,500 (for kHz)\n");
          printf("                           1,2,4,8,16 (for MHz)\n");
          printf("                adbit ---- A/D bit\n");
          printf("                           1,2,4,8\n");
          printf("                          default is 1\n");
          printf("                numch ---- number of channel\n");
          printf("                           1,4\n");
          printf("                          default is 1\n");
          printf("                filename ---- out file name\n");
          printf("                          default is tds.data\n");
          printf("                logfile ---- log out file name\n");
          printf("                          default is sampling.log\n");
          printf("                          if default logfile is used, it is overwritten.\n");
          printf("                          But if logfile is set by ""logfile"" parameter,\n");
          printf("                          log will be appended to logfile.\n");
          exit(0);
    }

    logmode=0;  // log file is first cleared
    spg.f_size_mode=1;    //1: divided into 2GB size file  0: no limitation
    span = atoi(*(argv + 1 ));
    if(span < 0 ){
       span=-span;
       spg.f_size_mode=0;
    }
    sfreq= atoi(*(argv + 2 ));
    if (argc>=4){
       adbit = atoi(*(argv + 3 ));
    } else {
          adbit=1;
    }

    if (argc>=5){
       numch = atoi(*(argv + 4 ));
    } else {
          numch=1;
    }

    if (argc>=6){
          strcpy(outfile,*(argv+5));
    } else {
          strcpy(outfile,dflt_fname);
    }
    if (argc>=7){
          strcpy(logfile,*(argv+6));
          logmode=1; // log is appended to existed log file
    } else {
          strcpy(logfile,dflt_logfile);
    }
    /* log info set if necessary */
    strcpy(lg.logfile,logfile);
    lg.hostname[0]='\0';
    lg.logdir[0]='\0';
    //lg.logfiletype=-1;    //-1: no log   0: log out
    lg.logfiletype=0;    //-1: no log   0: log out
    spg.lg=lg;
    if(logmode==0){
       sprintf(cbuf,"rm -f %s\0",logfile);
       system(cbuf);    // delete logfile
    }

    printf(" span    = %d\n",span);    // debug
    printf(" sfreq   = %d\n",sfreq);   // debug
    printf(" adbit   = %d\n",adbit);   // debug
    printf(" numch   = %d\n",numch);   // debug
    printf(" outfile = %s\n",outfile);   // debug
    if(lg.logfiletype==0){
    printf(" logfile = %s\n",logfile);   // debug
    }
    sprintf(cbuf,"sampling start: span=%d sfreq=%d adbit=%d numch=%d outfile=%s\0",
                 span,sfreq,adbit,numch,outfile);
    LogWrite4(&lg, 0, 0.0, 0, cbuf);

    spg.span=span;
    spg.sfreq=sfreq;
    spg.adbit=adbit;
    spg.numch=numch;
    strcpy(spg.outfile,outfile);
    //spg.f_size_mode=1;    //1: divided into 2GB size file  0: no limitation
    
    rv=sampleget(&spg);
    sprintf(cbuf,"sampling stop\0");
    LogWrite4(&lg, 0, 0.0, 0, cbuf);
    if(rv > 0) {
       sprintf(cbuf,"read error occured at least once (%d)",rv);
       LogWrite4(&lg, 0, 0.0, 1, cbuf);
    }
    printf("Data File is %s\n",outfile);
    if(lg.logfiletype==0){
       printf("Log  file is %s\n",spg.lg.logfile);
    }

    return 0;
}
